@@ -16,13 +16,9 @@
 
 #include "sox_i.h"
 #include "aiff.h"
+#include "id3.h"
 
 #include <time.h>      /* for time stamping comments */
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <errno.h>
-#include <limits.h>
 
 /* forward declarations */
 static double read_ieee_extended(sox_format_t *);
@@ -76,15 +72,15 @@ int lsx_aiffstartread(sox_format_t * ft)
 }
 
   /* FORM chunk */
-  if (lsx_reads(ft, buf, (size_t)4) == SOX_EOF || strncmp(buf, "FORM", (size_t)4) != 0) {
-    lsx_fail_errno(ft,SOX_EHDR,"AIFF header does not begin with magic word `FORM'");
+  if (lsx_reads(ft, buf, (size_t)4) || strncmp(buf, "FORM", (size_t)4) != 0) {
+    lsx_fail_errno(ft,SOX_EHDR,"header does not begin with magic word `FORM'");
     return(SOX_EOF);
   }
   if (lsx_readdw(ft, &totalsize))
     return(SOX_EOF);
-  if (lsx_reads(ft, buf, (size_t)4) == SOX_EOF || (strncmp(buf, "AIFF", (size_t)4) != 0 &&
+  if (lsx_reads(ft, buf, (size_t)4) || (strncmp(buf, "AIFF", (size_t)4) != 0 &&
         strncmp(buf, "AIFC", (size_t)4) != 0)) {
-    lsx_fail_errno(ft,SOX_EHDR,"AIFF `FORM' chunk does not specify `AIFF' or `AIFC' as type");
+    lsx_fail_errno(ft,SOX_EHDR,"`FORM' chunk does not specify `AIFF' or `AIFC' as type");
     return(SOX_EOF);
   }
 
@@ -93,10 +89,10 @@ int lsx_aiffstartread(sox_format_t * ft)
   /* The SSND chunk must be the last in the file */
   while (1) {
     if (lsx_reads(ft, buf, (size_t)4)) {
-      if (seekto > 0)
+      if (ssndsize > 0)
         break;
       else {
-        lsx_fail_errno(ft,SOX_EHDR,"Missing SSND chunk in AIFF file");
+        lsx_fail_errno(ft,SOX_EHDR,"missing SSND chunk");
         return(SOX_EOF);
       }
     }
@@ -136,10 +132,30 @@ int lsx_aiffstartread(sox_format_t * ft)
             return SOX_EOF;
           }
         }
+        else if (strncmp(buf, "alaw", (size_t)4) == 0 ||
+            strncmp(buf, "ALAW", (size_t)4) == 0) {
+          enc = SOX_ENCODING_ALAW;
+          if (bits != 16) {
+            lsx_fail_errno(ft, SOX_EHDR,
+              "Sample size of %u is not consistent with `ALAW' compression type", bits);
+            return SOX_EOF;
+          }
+	  bits = 8;
+        }
+        else if (strncmp(buf, "ulaw", (size_t)4) == 0 ||
+            strncmp(buf, "ULAW", (size_t)4) == 0) {
+          enc = SOX_ENCODING_ULAW;
+          if (bits != 16) {
+            lsx_fail_errno(ft, SOX_EHDR,
+              "Sample size of %u is not consistent with `ULAW' compression type", bits);
+            return SOX_EOF;
+          }
+	  bits = 8;
+        }
         else if (strncmp(buf, "NONE", (size_t)4) != 0 &&
             strncmp(buf, "twos", (size_t)4) != 0) {
           buf[4] = 0;
-          lsx_fail_errno(ft, SOX_EHDR, "Unsupported AIFC compression type `%s'", buf);
+          lsx_fail_errno(ft, SOX_EHDR, "unsupported AIFC compression type `%s'", buf);
           return(SOX_EOF);
         }
       }
@@ -326,6 +342,13 @@ int lsx_aiffstartread(sox_format_t * ft)
       }
       free(copyright);
     }
+    else if (strncmp(buf, "ID3 ", (size_t)4) == 0) {
+      off_t offs;
+      lsx_readdw(ft, &chunksize);
+      offs = lsx_tell(ft);
+      lsx_id3_read_tag(ft, 0);
+      lsx_seeki(ft, offs + chunksize, SEEK_SET);
+    }
     else {
       if (lsx_eof(ft))
         break;
@@ -359,7 +382,7 @@ int lsx_aiffstartread(sox_format_t * ft)
         return(SOX_EOF);
       }
     } else {
-      lsx_fail_errno(ft,SOX_EOF,"AIFF: no sound data on input file");
+      lsx_fail_errno(ft,SOX_EOF,"no sound data on input file");
       return(SOX_EOF);
     }
   }
@@ -370,33 +393,44 @@ int lsx_aiffstartread(sox_format_t * ft)
   ssndsize -= offset;
   while (offset-- > 0) {
     if (lsx_readb(ft, &trash8)) {
-      lsx_fail_errno(ft,errno,"unexpected EOF while skipping AIFF offset");
+      lsx_fail_errno(ft,errno,"unexpected EOF while skipping offset");
       return(SOX_EOF);
     }
   }
 
   if (foundcomm) {
-    if      (bits <=  8) bits = 8;
-    else if (bits <= 16) bits = 16;
-    else if (bits <= 24) bits = 24;
-    else if (bits <= 32) bits = 32;
-    else if (bits == 64 && enc == SOX_ENCODING_FLOAT) /* no-op */;
-    else {
-      lsx_fail_errno(ft,SOX_EFMT,"unsupported sample size in AIFF header: %d", bits);
-      return(SOX_EOF);
+    switch (enc) {
+    case SOX_ENCODING_SIGN2:
+	if      (bits <=  8) bits = 8;
+	else if (bits <= 16) bits = 16;
+	else if (bits <= 24) bits = 24;
+	else if (bits <= 32) bits = 32;
+	goto OK;
+    case SOX_ENCODING_FLOAT:
+        if (bits == 32 || bits == 64) goto OK;
+	break;
+    case SOX_ENCODING_ALAW:
+    case SOX_ENCODING_ULAW:
+        if (bits == 8) goto OK;
+	break;
+    default: /* can't happen */
+	lsx_fail("internal error in lsx_aiffstartread");
+	return(SOX_EOF);
     }
-  } else  {
+    lsx_fail_errno(ft,SOX_EFMT,"unsupported sample size in header: %d", bits);
+    return(SOX_EOF);
+  } else {
     if ((ft->signal.channels == SOX_UNSPEC)
         || (ft->signal.rate == SOX_UNSPEC)
         || (ft->encoding.encoding == SOX_ENCODING_UNKNOWN)
         || (ft->encoding.bits_per_sample == 0)) {
       lsx_report("You must specify # channels, sample rate, signed/unsigned,");
-      lsx_report("and 8/16 on the command line.");
-      lsx_fail_errno(ft,SOX_EFMT,"Bogus AIFF file: no COMM section.");
+      lsx_report("and -b 8/16 on the command line.");
+      lsx_fail_errno(ft,SOX_EFMT,"bogus file: no COMM section");
       return(SOX_EOF);
     }
-
   }
+OK:
   ssndsize /= bits >> 3;
 
   /* Cope with 'sowt' CD tracks as read on Macs */
@@ -498,7 +532,7 @@ static int textChunk(char **text, char *chunkDescription, sox_format_t * ft)
 
   if (lsx_readbuf(ft, *text, (size_t) chunksize) != chunksize)
   {
-    lsx_fail_errno(ft,SOX_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
+    lsx_fail_errno(ft,SOX_EOF,"unexpected EOF in %s header", chunkDescription);
     return(SOX_EOF);
   }
   if (chunksize != SOX_SIZE_MAX)
@@ -511,7 +545,7 @@ static int textChunk(char **text, char *chunkDescription, sox_format_t * ft)
     char c;
     if (lsx_readbuf(ft, &c, (size_t)1) != 1)
     {
-      lsx_fail_errno(ft,SOX_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
+      lsx_fail_errno(ft,SOX_EOF,"unexpected EOF in %s header", chunkDescription);
       return(SOX_EOF);
     }
   }
@@ -544,7 +578,7 @@ static int commentChunk(char **text, char *chunkDescription, sox_format_t * ft)
         lsx_readw(ft, &commentLength))
       return SOX_EOF;
     if (((size_t)totalCommentLength) + commentLength > USHRT_MAX) {
-        lsx_fail_errno(ft,SOX_EOF,"AIFF: Comment too long in %s header", chunkDescription);
+        lsx_fail_errno(ft,SOX_EOF,"comment too long in %s header", chunkDescription);
         return(SOX_EOF);
     }
     totalCommentLength += commentLength;
@@ -557,7 +591,7 @@ static int commentChunk(char **text, char *chunkDescription, sox_format_t * ft)
     }
 
     if (lsx_readbuf(ft, *text + totalCommentLength - commentLength, (size_t) commentLength) != commentLength) {
-        lsx_fail_errno(ft,SOX_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
+        lsx_fail_errno(ft,SOX_EOF,"unexpected EOF in %s header", chunkDescription);
         return(SOX_EOF);
     }
     *(*text + totalCommentLength) = '\0';
@@ -566,7 +600,7 @@ static int commentChunk(char **text, char *chunkDescription, sox_format_t * ft)
         /* Read past pad byte */
         char c;
         if (lsx_readbuf(ft, &c, (size_t)1) != 1) {
-            lsx_fail_errno(ft,SOX_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
+            lsx_fail_errno(ft,SOX_EOF,"unexpected EOF in %s header", chunkDescription);
             return(SOX_EOF);
         }
         totalReadLength += 1;
@@ -642,7 +676,7 @@ int lsx_aiffstartwrite(sox_format_t * ft)
            number of samples. */
         if (ft->signal.channels >= (0x7f000000 / (ft->encoding.bits_per_sample >> 3)))
         {
-                lsx_fail_errno(ft, SOX_EOF, "too many channels for AIFF header");
+                lsx_fail_errno(ft, SOX_EOF, "too many channels for header");
                 return SOX_EOF;
         }
         return(aiffwriteheader(ft, (uint64_t) 0x7f000000 / ((ft->encoding.bits_per_sample>>3)*ft->signal.channels)));
@@ -660,12 +694,12 @@ int lsx_aiffstopwrite(sox_format_t * ft)
 
         if (!ft->seekable)
         {
-            lsx_fail_errno(ft,SOX_EOF,"Non-seekable file.");
+            lsx_fail_errno(ft,SOX_EOF,"non-seekable file");
             return(SOX_EOF);
         }
         if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
         {
-                lsx_fail_errno(ft,errno,"can't rewind output file to rewrite AIFF header");
+                lsx_fail_errno(ft,errno,"can't rewind output file to rewrite header");
                 return(SOX_EOF);
         }
         return(aiffwriteheader(ft, ft->olength / ft->signal.channels));
@@ -762,7 +796,7 @@ static int aiffwriteheader(sox_format_t * ft, uint64_t nframes)
                 bits = 32;
         else
         {
-                lsx_fail_errno(ft,SOX_EFMT,"unsupported output encoding/size for AIFF header");
+                lsx_fail_errno(ft,SOX_EFMT,"unsupported output encoding/size");
                 return(SOX_EOF);
         }
 
@@ -867,12 +901,12 @@ int lsx_aifcstopwrite(sox_format_t * ft)
 
         if (!ft->seekable)
         {
-            lsx_fail_errno(ft,SOX_EOF,"Non-seekable file.");
+            lsx_fail_errno(ft,SOX_EOF,"non-seekable file");
             return(SOX_EOF);
         }
         if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
         {
-                lsx_fail_errno(ft,errno,"can't rewind output file to rewrite AIFC header");
+                lsx_fail_errno(ft,errno,"can't rewind output file to rewrite header");
                 return(SOX_EOF);
         }
         return(aifcwriteheader(ft, ft->olength / ft->signal.channels));
@@ -904,9 +938,15 @@ static int aifcwriteheader(sox_format_t * ft, uint64_t nframes)
         else if (ft->encoding.encoding == SOX_ENCODING_FLOAT &&
                  ft->encoding.bits_per_sample == 64)
                 bits = 64;
+        else if (ft->encoding.encoding == SOX_ENCODING_ALAW &&
+                 ft->encoding.bits_per_sample == 8)
+                bits = 16;
+        else if (ft->encoding.encoding == SOX_ENCODING_ULAW &&
+                 ft->encoding.bits_per_sample == 8)
+                bits = 16;
         else
         {
-                lsx_fail_errno(ft,SOX_EFMT,"unsupported output encoding/size for AIFC header");
+                lsx_fail_errno(ft,SOX_EFMT,"unsupported output encoding/size");
                 return(SOX_EOF);
         }
 
@@ -925,8 +965,17 @@ static int aifcwriteheader(sox_format_t * ft, uint64_t nframes)
               cname = "64-bit floating point";
             }
             break;
+          case SOX_ENCODING_ALAW:
+	    ctype = "alaw";
+	    cname = "8-bit A-law";
+	    break;
+          case SOX_ENCODING_ULAW:
+	    ctype = "ulaw";
+	    cname = "8-bit mu-law";
+	    break;
           default: /* can't happen */
-            break;
+            lsx_fail("internal error in aifcwriteheader");
+	    return(SOX_EOF);
         }
         cname_len = strlen(cname);
         comm_len = 18+4+1+cname_len;
@@ -971,13 +1020,14 @@ static int aifcwriteheader(sox_format_t * ft, uint64_t nframes)
             lsx_writes(ft, cname) ||
             (comm_padding ? lsx_writeb(ft, 0) : 0) ||
 
+	    write_mark_and_inst_chunks(ft) ||
+
             /* SSND chunk -- describes data */
             lsx_writes(ft, "SSND") ||
             /* chunk size */
             lsx_writedw(ft, (unsigned) (8 + nframes * ft->signal.channels * (ft->encoding.bits_per_sample >> 3))) ||
             lsx_writedw(ft, 0) || /* offset */
-            lsx_writedw(ft, 0) || /* block size */
-	    write_mark_and_inst_chunks(ft))
+            lsx_writedw(ft, 0)) /* block size */
 	        return(SOX_EOF);
 
         /* Any Private chunks shall appear after the required chunks (FORM,FVER,COMM,SSND) */
