@@ -18,16 +18,28 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/* References:
+ * https://www.garykessler.net/library/file_sigs.html
+ * https://en.wikipedia.org/wiki/List_of_file_signatures
+ *
+ * Sources of audio files in unusual formats:
+ * http://www-mmsp.ece.mcgill.ca/Documents/AudioFormats
+ */
+
 #include "sox_i.h"
 
-#include <assert.h>
 #include <ctype.h>
-#include <errno.h>
 #include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/stat.h>
-#include <sys/types.h>
+
+#ifdef __ANDROID__
+  /* From Termux to "Get rid of references to leaked builtins"
+   * https://github.com/termux/termux-packages/tree/858771625/packages/sox */
+  #include <bits/struct_file.h>
+#endif
+
+#ifdef __ANDROID__
+  #include <bits/struct_file.h>
+#endif
 
 #ifdef HAVE_IO_H
   #include <io.h>
@@ -37,13 +49,22 @@
   #include <magic.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_WAIT_H
+#  include <sys/wait.h>	/* for WEXITSTATUS */
+#endif
+
 #define PIPE_AUTO_DETECT_SIZE 256 /* Only as much as we can rewind a pipe */
 #define AUTO_DETECT_SIZE 4096     /* For seekable file, so no restriction */
 
 static char const * auto_detect_format(sox_format_t * ft, char const * ext)
 {
   char data[AUTO_DETECT_SIZE];
-  size_t len = lsx_readbuf(ft, data, ft->seekable? sizeof(data) : PIPE_AUTO_DETECT_SIZE);
+  size_t len = lsx_readbuf_rewind(ft, data, ft->seekable ? sizeof(data) : PIPE_AUTO_DETECT_SIZE);
+
   #define CHECK(type, p2, l2, d2, p1, l1, d1) if (len >= p1 + l1 && \
       !memcmp(data + p1, d1, (size_t)l1) && !memcmp(data + p2, d2, (size_t)l2)) return #type;
   CHECK(voc   , 0, 0, ""     , 0, 20, "Creative Voice File\x1a")
@@ -58,7 +79,6 @@ static char const * auto_detect_format(sox_format_t * ft, char const * ext)
   CHECK(sndt  , 0, 0, ""     , 0,  6, "SOUND\x1a")
   CHECK(vorbis, 0, 4, "OggS" , 29, 6, "vorbis")
   CHECK(opus  , 0, 4, "OggS" , 28, 8, "OpusHead")
-  CHECK(speex , 0, 4, "OggS" , 28, 6, "Speex")
   CHECK(hcom  ,65, 4, "FSSD" , 128,4, "HCOM")
   CHECK(wav   , 0, 4, "RIFF" , 8,  4, "WAVE")
   CHECK(wav   , 0, 4, "RIFX" , 8,  4, "WAVE")
@@ -67,6 +87,7 @@ static char const * auto_detect_format(sox_format_t * ft, char const * ext)
   CHECK(aifc  , 0, 4, "FORM" , 8,  4, "AIFC")
   CHECK(8svx  , 0, 4, "FORM" , 8,  4, "8SVX")
   CHECK(maud  , 0, 4, "FORM" , 8,  4, "MAUD")
+  CHECK(nsp   , 0, 4, "FORM" , 4,  4, "DS16")
   CHECK(xa    , 0, 0, ""     , 0,  4, "XA\0\0")
   CHECK(xa    , 0, 0, ""     , 0,  4, "XAI\0")
   CHECK(xa    , 0, 0, ""     , 0,  4, "XAJ\0")
@@ -88,6 +109,110 @@ static char const * auto_detect_format(sox_format_t * ft, char const * ext)
   CHECK(sf    , 0, 0, ""     , 0,  4, "\144\243\004\0")
   CHECK(sox   , 0, 0, ""     , 0,  4, ".SoX")
   CHECK(sox   , 0, 0, ""     , 0,  4, "XoS.")
+  CHECK(mp3   , 0, 0, ""     , 0,  3, "ID3")
+  /* First 16 bits for MPEG 1 layer 3 */
+  CHECK(mp3   , 0, 0, ""     , 0,  2, "\xFF\xFA") /* CRC protected */
+  CHECK(mp3   , 0, 0, ""     , 0,  2, "\xFF\xFB") /* Not protected */
+  /* First 16 bits for MPEG 1 layer 2 */
+  CHECK(mp2   , 0, 0, ""     , 0,  2, "\xFF\xFC") /* CRC protected */
+  CHECK(mp2   , 0, 0, ""     , 0,  2, "\xFF\xFD") /* Not protected */
+
+#if HAVE_SNDFILE
+  CHECK(sds   , 0, 0, ""     , 0,  2, "\xF0\x7E")
+# if HAVE_SF_FORMAT_MPC2K
+  CHECK(mpc2k , 0, 0, ""     , 0,  2, "\x01\x04")
+	/* sndfile always writes [100,0] at [19] */
+# endif
+#endif
+
+#if USING_FFMPEG
+  CHECK(3g2   , 0, 0, ""     , 4,  8, "ftyp3g2a")
+  CHECK(3g2   , 0, 0, ""     , 4,  8, "ftyp3g2b")
+  CHECK(3gp   , 0, 0, ""     , 4,  8, "ftyp3gp6")
+  CHECK(3gp   , 0, 0, ""     , 4,  8, "ftyp3gp4")
+  CHECK(3gp   , 0, 0, ""     , 4,  6, "ftyp3g")
+  CHECK(aac   , 0, 0, ""     , 0,  2, "\xFF\xF1")
+  CHECK(aac   , 0, 0, ""     , 0,  2, "\xFF\xF9")
+  CHECK(ac3   , 0, 0, ""     , 0,  2, "\x0B\x77")
+  /* adts: By inspection of ffmpeg output for many files with all options */
+  CHECK(adts  , 0, 8, "\x49\x44\x33\x04\x00\x00\x00\x00"  /* -write_id3v2 */
+                             , 14, 3, "\x00\x00\x00")
+  CHECK(adts  , 0, 2, "\xFF\xF1"		/* no tag or -write_apetag */
+                             ,  6,18, "\xFC\xDE\x02\x00\x4C\x61\x76\x63\x35\x39\x2E\x33\x37\x2E\x31\x30\x30\x00")
+  CHECK(adts  , 0, 2, "\xFF\xF9"			  /* -write_mpeg2 */
+                             ,  6,18, "\xFC\xDE\x02\x00\x4C\x61\x76\x63\x35\x39\x2E\x33\x37\x2E\x31\x30\x30\x00")
+  /* adx format starts with \x80\x00 but that gives false positives too often
+   * and makes "tests.sh" fail on bigendian hosts */
+#if 0
+  CHECK(adx   , 0, 0, ""     , 0,  2, "\x80\x00") /* libavformat/adxdec.c */
+#endif
+  CHECK(ape   , 0, 0, ""     , 0,  4, "MAC ")     /* libavformat/ape.c */
+  CHECK(apm   ,20, 4, "vs12" , 0,  2, "\x00\x20") /* libavformat/apm.c */
+  /* aptx is headerless and can only be autodetected by the filename extension */
+  /* libavformat/argo_asf.[ch] */
+  CHECK(argo_asf,0,0, ""     , 0,  8, "ASF\x00\x01\x00\x01\x00")
+  CHECK(argo_asf,0,0, ""     , 0,  8, "ASF\x00\x01\x00\x02\x00")
+  CHECK(argo_asf,0,0, ""     , 0,  8, "ASF\x00\x02\x00\x01\x00")
+  CHECK(asf   , 0, 0, ""     , 0, 16, "\x30\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C") /* libavformat/asfdec_*.c */
+  CHECK(ast   , 0, 0, ""     , 0,  4, "STRM")    /* libavformat/astdec.c */
+  CHECK(avi   , 0, 4, "RIFF" , 8,  4, "AVI ")    /* libavformat/avidec.c */
+  CHECK(avi   , 0, 4, "RIFF" , 8,  4, "AVIX")
+  CHECK(avi   , 0, 4, "RIFF" , 8,  4, "AVI\x19")
+  CHECK(avi   , 0, 4, "ON2 " , 8,  4, "ON2f")
+  CHECK(avi   , 0, 4, "RIFF" , 8,  4, "AMV ")
+  /* dfpwm is headerless and can only be autodetected by the filename extension */
+  /* dts is autodetected by ffmpeg but not by fixed bytes at fixed offsets */
+  CHECK(eac3  , 0, 0, ""     , 0,  2, "\x04\x00") /* libavformat/eacdata.c */
+  CHECK(eac3  , 0, 0, ""     , 0,  2, "\x04\x04")
+  CHECK(eac3  , 0, 0, ""     , 0,  2, "\x04\x0C")
+  CHECK(eac3  , 0, 0, ""     , 0,  2, "\x04\x14")
+  CHECK(f4v   , 0, 0, ""     , 4,  8, "ftypf4v ") /* libavformat/movenc.c */
+  CHECK(flv   , 5, 1, "\x00" , 0,  4, "FLV\x00")  /* libavformat/flvdec.c */
+  CHECK(flv   , 5, 1, "\x00" , 0,  4, "FLV\x01")
+  CHECK(flv   , 5, 1, "\x00" , 0,  4, "FLV\x02")
+  CHECK(flv   , 5, 1, "\x00" , 0,  4, "FLV\x03")
+  CHECK(flv   , 5, 1, "\x00" , 0,  4, "FLV\x04")
+  CHECK(gxf   ,10, 6, "\x00\x00\x00\x00\xE1\xE2", 0, 6, "\x00\x00\x00\x00\x01\xBC") /* libavformat/gxf.c */
+  CHECK(ism   , 0, 0, ""     , 4,  8, "ftypisml")
+  CHECK(kvag  , 0, 0, ""     , 0,  4, "KVAG")	 /* libavformat/kvag.c */
+  CHECK(m4a   , 0, 0, ""     , 4,  8, "ftypM4A ") /* iPod format */
+  CHECK(m4v   , 0, 0, ""     , 4,  8, "ftypM4V ") /* iPod format */
+  CHECK(mkv   , 0, 0, ""     , 0,  4, "\x1A\x45\xDF\xA3")
+  CHECK(mlp   , 0, 0, ""     , 4,  4, "\xF8\x72\x6F\xBB")
+  CHECK(mov   , 0, 0, ""     , 4,  8, "ftypavif")
+  CHECK(mov   , 0, 0, ""     , 4,  8, "ftypavis") /* Animated AVIF */
+  CHECK(mov   , 0, 0, ""     , 4,  8, "ftypqt  ")
+  CHECK(mov   , 0, 0, ""     , 4,  8, "ftypMSNV") /* PSP format */
+  CHECK(mp4   , 0, 0, ""     , 4,  8, "ftypisom")
+  CHECK(mp4   , 0, 0, ""     , 4,  8, "ftypiso4")
+  CHECK(mp4   , 0, 0, ""     , 4,  8, "ftypiso5")
+  CHECK(mp4   , 0, 0, ""     , 4,  8, "ftypiso6")
+  /* mpeg by inspection of ffmpeg -f vcd/svcd/vob/dvd output */
+  CHECK(mpeg  , 0, 0, ""     , 0,  4, "\x00\x00\x01\xBA")
+  /* MPEG Transport Streams have a sync byte every 188 bytes */
+  CHECK(mpegts, 0, 1, "\x47" ,188, 1, "\x47")
+  CHECK(mxf_opatom,0,0,""    , 0,  4, "\x06\x0E\x2B\x34")
+  CHECK(nut   , 0, 0, ""     , 0, 25, "nut/multimedia container\0")
+  CHECK(oga   , 0, 0, ""     , 0, 14, "OggS\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00")
+  CHECK(ra    , 0, 0, ""     , 0,  9, "\x2E\x52\x4D\x46\x00\x00\x00\x12\x00")
+  CHECK(rm    , 0, 0, ""     , 0,  6, ".RMF\0\0") /* libavformat/rmdec.c */
+  CHECK(rm    , 0, 0, ""     , 0,  4, ".ra\xFD")
+  /* rso can't be autodetected by ffmpeg */
+  /* sbc can't be autodetected by ffmpeg */
+  CHECK(smjpeg,0, 0, ""     , 0,  8, "\x0\xaSMJPEG") /* libavformat/smjpeg.h */
+  /* spdif is autodetected by ffmpeg but not by fixed bytes at fixed offsets */
+  CHECK(spx   , 0, 4, "OggS" , 28, 5, "Speex")
+  CHECK(tta   , 4, 2, "\1\0" , 0,  4, "TTA1") /* libavformat/tta.c */
+  CHECK(tta   , 4, 2, "\2\0" , 0,  4, "TTA1")
+  CHECK(vag   , 0, 0, ""     , 0,  7, "VAGp\0\0\0")
+  CHECK(wma   , 0, 0, ""     , 0, 16, "\x30\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C")
+  /* wsaud has no file signature but there are header sanity checks
+   * and the header signature of the first audio chunk.
+   * libavformat/westwood_aud.c */
+  CHECK(wsaud ,11, 1, "\1"   ,16,  4, "\xAF\xDE\x00\x00")
+  CHECK(wsaud ,11, 1, "\x63" ,16,  4, "\xAF\xDE\x00\x00")
+  CHECK(wtv   , 0, 0, ""     , 0, 16, "\xB7\xD8\x00\x20\x37\x49\xDA\x11\xA6\x4E\x00\x07\xE9\x5E\xAD\x8D") /* libavformat/wtv_common.c */
+#endif /* USING_FFMPEG */
 
   if (ext && !strcasecmp(ext, "snd"))
   CHECK(sndr  , 7, 1, ""     , 0,  2, "\0")
@@ -145,6 +270,7 @@ static sox_encodings_info_t const s_sox_encodings_info[] = {
   {sox_encodings_lossy2, "CVSD"         , "CVSD"},
   {sox_encodings_lossy2, "LPC10"        , "LPC10"},
   {sox_encodings_lossy2, "Opus"         , "Opus"},
+  {sox_encodings_none  , "DSD"          , "Direct Stream Digital"},
 };
 
 assert_static(array_length(s_sox_encodings_info) == SOX_ENCODINGS,
@@ -169,6 +295,7 @@ unsigned sox_precision(sox_encoding_t encoding, unsigned bits_per_sample)
 
     case SOX_ENCODING_ALAW:       return bits_per_sample == 8? 13: 0;
     case SOX_ENCODING_ULAW:       return bits_per_sample == 8? 14: 0;
+    case SOX_ENCODING_DSD:        return bits_per_sample;
 
     case SOX_ENCODING_CL_ADPCM:   return bits_per_sample? 8: 0;
     case SOX_ENCODING_CL_ADPCM16: return bits_per_sample == 4? 13: 0;
@@ -223,7 +350,7 @@ size_t sox_num_comments(sox_comments_t comments)
 void sox_append_comment(sox_comments_t * comments, char const * comment)
 {
   size_t n = sox_num_comments(*comments);
-  *comments = lsx_realloc(*comments, (n + 2) * sizeof(**comments));
+  lsx_revalloc(*comments, n + 2);
   assert(comment);
   (*comments)[n++] = lsx_strdup(comment);
   (*comments)[n] = 0;
@@ -235,7 +362,10 @@ void sox_append_comments(sox_comments_t * comments, char const * comment)
   if (comment) {
     while ((end = strchr(comment, '\n'))) {
       size_t len = end - comment;
-      char * c = lsx_malloc((len + 1) * sizeof(*c));
+      char * c;
+
+      lsx_valloc(c, len + 1);
+
       strncpy(c, comment, len);
       c[len] = '\0';
       sox_append_comment(comments, c);
@@ -275,7 +405,7 @@ char * lsx_cat_comments(sox_comments_t comments)
   if (p) while (*p)
     len += strlen(*p++) + 1;
 
-  result = lsx_calloc(len? len : 1, sizeof(*result));
+  lsx_vcalloc(result, len? len : 1);
 
   if ((p = comments) && *p) {
     strcpy(result, *p);
@@ -331,12 +461,26 @@ static void set_endiannesses(sox_format_t * ft)
 static sox_bool is_seekable(sox_format_t const * ft)
 {
   struct stat st;
+  int fd, seekable;
 
   assert(ft);
   if (!ft->fp)
     return sox_false;
-  fstat(fileno((FILE*)ft->fp), &st);
-  return ((st.st_mode & S_IFMT) == S_IFREG);
+  fd = fileno((FILE*)ft->fp);
+  if (fd < 0)
+     return 0;
+  fstat(fd, &st);
+  seekable = ((st.st_mode & S_IFMT) == S_IFREG);
+#if defined HAVE_POSIX_FADVISE && defined POSIX_FADV_SEQUENTIAL
+  if (seekable) {
+    /*
+     * POSIX_FADV_NOREUSE can potentially be beneficial, too,
+     * but is a no-op as of Linux 4.2.  Not sure about other kernels.
+     */
+    (void)posix_fadvise(fd, (off_t)0, st.st_size, POSIX_FADV_SEQUENTIAL);
+  }
+#endif
+  return seekable;
 }
 
 /* check that all settings have been given */
@@ -344,8 +488,8 @@ static int sox_checkformat(sox_format_t * ft)
 {
   ft->sox_errno = SOX_SUCCESS;
 
-  if (ft->signal.rate <= 0) {
-    lsx_fail_errno(ft, SOX_EFMT, "sample rate zero or negative");
+  if (!ft->signal.rate) {
+    lsx_fail_errno(ft,SOX_EFMT,"sampling rate was not specified");
     return SOX_EOF;
   }
   if (!ft->signal.precision) {
@@ -355,7 +499,7 @@ static int sox_checkformat(sox_format_t * ft)
   return SOX_SUCCESS;
 }
 
-static sox_bool is_url(char const * text) /* detects only wget-supported URLs */
+static sox_bool is_url(char const * text)
 {
   return !(
       strncasecmp(text, "http:" , (size_t)5) &&
@@ -365,70 +509,299 @@ static sox_bool is_url(char const * text) /* detects only wget-supported URLs */
 
 static int xfclose(FILE * file, lsx_io_type io_type)
 {
-  return
-#ifdef HAVE_POPEN
-    io_type != lsx_io_file? pclose(file) :
+  if (file == NULL) return SOX_SUCCESS;  /* Shouldn't happen */
+#if HAVE_POPEN
+  return io_type != lsx_io_file ? pclose(file) : fclose(file);
+#else
+  (void) io_type;
+  return fclose(file);
+#endif
+}
+
+#if HAVE_POPEN
+static void incr_pipe_size(FILE *f)
+{
+/*
+ * Linux 2.6.35 and later has the ability to expand the pipe buffer
+ * Try to get it as big as possible to avoid stalls when SoX itself
+ * is using big buffers
+ */
+#ifdef F_SETPIPE_SZ
+  static long max_pipe_size;
+
+  /* read the maximum size of the pipe the first time this is called */
+  if (max_pipe_size == 0) {
+    const char path[] = "/proc/sys/fs/pipe-max-size";
+    int fd = open(path, O_RDONLY);
+
+    max_pipe_size = -1;
+    if (fd >= 0) {
+      char buf[80];
+      ssize_t r = read(fd, buf, sizeof(buf));
+
+      if (r > 0) {
+        buf[r] = 0;
+        max_pipe_size = strtol(buf, NULL, 10);
+
+        /* guard against obviously wrong values on messed up systems */
+        if (max_pipe_size <= PIPE_BUF || max_pipe_size > INT_MAX)
+          max_pipe_size = -1;
+      }
+      close(fd);
+    }
+  }
+
+  if (max_pipe_size > PIPE_BUF) {
+    int fd = fileno(f);
+
+    if (fcntl(fd, F_SETPIPE_SZ, max_pipe_size) >= 0)
+      lsx_debug("got pipe %ld bytes\n", max_pipe_size);
+    else
+      lsx_warn("couldn't set pipe size to %ld bytes: %s\n",
+               max_pipe_size, strerror(errno));
+  }
+#else
+  /* do nothing for platforms without F_SETPIPE_SZ */
+  (void) f;
+#endif
+}
+#endif /* HAVE_POPEN */
+
+#ifndef POPEN_MODE
+# ifdef _WIN32
+#  define POPEN_MODE "rb"
+# else
+#  define POPEN_MODE "r"
+# endif
+#endif
+
+static FILE * open_url(char const * identifier)
+{
+#if !HAVE_POPEN
+    lsx_fail("this build of SoX cannot open URLs");
+    (void) identifier;
+    return NULL;
+#else
+    FILE *f;	/* The file descriptor to read from the pipe */
+    static const char * const command_args[][2] = {
+    /* Try wget before wget2 as it's more likely to be installed
+     * unless configured --with-curl
+     */
+# if USING_CURL
+	{ "curl",  "-f --no-cert-status -s -o -" },
+	{ "wget",  "--no-check-certificate -q -O -" },
+	{ "wget2", "--no-check-certificate -q -O -" },
+# else
+	{ "wget",  "--no-check-certificate -q -O -" },
+	{ "wget2", "--no-check-certificate -q -O -" },
+	{ "curl",  "-f --no-cert-status -s -o -" },
+# endif
+    };
+    static const char shutup[] =
+# ifdef _WIN32
+        "> NUL";
+# else
+        "> /dev/null 2>&1"; /* 2>&1 prevents "sh:  1: wget: not found." */
+# endif
+    char * command;
+    unsigned i;
+
+    /* See which of wget, wget2 and curl are installed
+     *
+     * You must send, at least, wget's output into the null device
+     * because its --version text is >1024 bytes and the pipe breaks
+     * making it look like wget is not installed.
+     */
+    command = NULL;
+    for (i = 0; i < sizeof(command_args) / sizeof(*command_args); i++) {
+        char tryit[36]; /* >= 5 + 1 + 9 + 1 + 16 + 1 == 33 */
+        sprintf(tryit, "%s --version %s", command_args[i][0], shutup);
+        f = popen(tryit, POPEN_MODE);
+        if (f && pclose(f) == 0) {
+	   /* This is actually slightly longer because
+	    * we must add a nul but lose several %sses */
+           command = lsx_malloc(strlen(command_args[i][0]) +
+	                        strlen(command_args[i][1]) +
+				strlen(identifier));
+           sprintf(command, "%s %s \"%s\"", command_args[i][0],
+					    command_args[i][1],
+	                                    identifier);
+	   break;
+        }
+    }
+    if (!command) {
+        lsx_fail("to read URLs Please install one of wget, wget2 and curl");
+	return NULL;
+    }
+
+    f = popen(command, POPEN_MODE);
+    if (f == NULL) {
+        lsx_fail("cannot popen `%s'", command);
+        free(command);
+        return f;
+    }
+
+    incr_pipe_size(f);
+
+    /* To detect failure and report why, instead of saying
+     * sox_ng FAIL formats: can't determine type of file URL `http://bla',
+     * try to read one byte.
+     */
+    {
+        int c;
+
+        if ((c = getc(f)) != EOF) {
+            (void) ungetc(c, f);
+            free(command);
+            return f;
+        }
+    }
+
+    /* We got EOF immediately so either it's a zero-length file
+     * or something went wrong. */
+    {
+	int status = pclose(f);
+        char const *s;
+
+#ifdef WEXITSTATUS
+        /* Some pclose()s return the command's exit status;
+         * others return a code that you have to decode with WEXITSTATUS
+         */
+        status = WEXITSTATUS(status);
+#endif
+	if (status == 0) {
+	    /* Nothing went wrong. It's a zero-length file. */
+	    s = "Zero length file";
+        } else if (status == -1) {
+	    /* Something went very wrong */
+	    s = strerror(errno);
+	} else if (strncmp(command, "wget", 4) == 0) switch (status) {
+	/* Decode the exit status into a meaning */
+	case 1: s = "Generic error code"; break;
+	case 2: s = "Parse error of command line/.wgetrc/.netrc"; break;
+	case 3: s = "File I/O error"; break;
+	case 4: s = "Network failure"; break;
+	case 5: s = "SSL verification failure"; break;
+	case 6: s = "Username/password authentication failure"; break;
+	case 7: s = "Protocol errors"; break;
+	case 8: s = "Server issued an error response"; break;
+	default: s = "Unrecognized exit code from wget"; break;
+	} else if (strncmp(command, "curl", 4) == 0) switch (status) {
+	case 1: s = "Unsupported protocol"; break;
+	case 2: s = "Failed to initialize."; break;
+	case 3: s = "URL malformed"; break;
+	case 4: s = "Feature was not enabled at build-time"; break;
+	case 5: s = "Could not resolve proxy"; break;
+	case 6: s = "Could not resolve host"; break;
+	case 7: s = "Failed to connect to host"; break;
+	case 8: s = "Weird server reply"; break;
+	case 9: s = "FTP access denied"; break;
+	case 10: s = "FTP accept failed"; break;
+	case 11: s = "FTP weird PASS reply"; break;
+	case 12: s = "FTP timeout"; break;
+	case 13: s = "FTP weird PASV reply"; break;
+	case 14: s = "FTP weird 227 format"; break;
+	case 15: s = "FTP cannot use host"; break;
+	case 16: s = "HTTP/2 error"; break;
+	case 17: s = "FTP could not set binary"; break;
+	case 18: s = "Partial file"; break;
+	case 19: s = "FTP RETR failed."; break;
+	case 21: s = "FTP quote error"; break;
+	case 22: s = "HTTP page not retrieved"; break;
+	case 23: s = "Write error"; break;
+	case 25: s = "FTP could not STOR file"; break;
+	case 26: s = "Read error"; break;
+	case 27: s = "Out of memory"; break;
+	case 28: s = "Operation timeout"; break;
+	case 30: s = "FTP PORT failed"; break;
+	case 31: s = "FTP could not use REST"; break;
+	case 33: s = "HTTP range error"; break;
+	case 34: s = "HTTP post error"; break;
+	case 35: s = "SSL connect error"; break;
+	case 36: s = "Bad download resume"; break;
+	case 37: s = "FILE could not read file"; break;
+	case 38: s = "LDAP cannot bind"; break;
+	case 39: s = "LDAP search failed"; break;
+	case 41: s = "Function not found"; break;
+	case 42: s = "Aborted by callback"; break;
+	case 43: s = "Internal error"; break;
+	case 45: s = "Interface error"; break;
+	case 47: s = "Too many redirects"; break;
+	case 48: s = "Unknown libcurl option"; break;
+	case 49: s = "Malformed telnet option"; break;
+	case 52: s = "Server did not reply"; break;
+	case 53: s = "SSL crypto engine not found"; break;
+	case 54: s = "Cannot set SSL crypto engine as default"; break;
+	case 55: s = "Failed sending network data"; break;
+	case 56: s = "Failure receiving network data"; break;
+	case 58: s = "Problem with local certificate"; break;
+	case 59: s = "Could not use specified SSL cipher"; break;
+	case 60: s = "Peer certificate cannot be authenticated"; break;
+	case 61: s = "Unrecognized transfer encoding"; break;
+	case 63: s = "Maximum file size exceeded"; break;
+	case 64: s = "FTP SSL level failed"; break;
+	case 65: s = "Rewind failed"; break;
+	case 66: s = "Failed to initialise SSL Engine"; break;
+	case 67: s = "Failed to log in"; break;
+	case 68: s = "File not found on TFTP server"; break;
+	case 69: s = "Permission problem on TFTP server"; break;
+	case 70: s = "Out of disk space on TFTP server"; break;
+	case 71: s = "Illegal TFTP operation"; break;
+	case 72: s = "Unknown TFTP transfer ID"; break;
+	case 73: s = "File already exists"; break;
+	case 74: s = "No such user"; break;
+	case 77: s = "Problem reading the SSL CA cert"; break;
+	case 78: s = "The resource referenced in the URL does not exist."; break;
+	case 79: s = "Unspecified error in SSH session"; break;
+	case 80: s = "Failed to shut down the SSL connection"; break;
+	case 82: s = "Could not load CRL file"; break;
+	case 83: s = "Issuer check failed"; break;
+	case 84: s = "The FTP PRET command failed"; break;
+	case 85: s = "RTSP CSeq numbers mismatch"; break;
+	case 86: s = "RTSP Session Identifiers mismatch"; break;
+	case 87: s = "Unable to parse FTP file list"; break;
+	case 88: s = "FTP chunk callback reported error"; break;
+	case 89: s = "No connection available"; break;
+	case 90: s = "SSL public key does not matched pinned public key"; break;
+	case 91: s = "Invalid SSL certificate status"; break;
+	case 92: s = "Stream error in HTTP/2 framing layer"; break;
+	case 93: s = "API function called from inside a callback"; break;
+	case 94: s = "Authentication function returned an error"; break;
+	case 95: s = "Problem in HTTP/3 layer"; break;
+	case 96: s = "QUIC connection error"; break;
+	default: s = "Unrecognized exit code from curl"; break;
+	} else s = "URL fetch failed";
+
+	/* And report the failure*/
+	lsx_fail("%s", s);
+	free(command);
+	return NULL;
+    }
 #endif
-    fclose(file);
 }
 
 static FILE * xfopen(char const * identifier, char const * mode, lsx_io_type * io_type)
 {
-  *io_type = lsx_io_file;
+  FILE * f = NULL;
 
   if (*identifier == '|') {
-    FILE * f = NULL;
-#ifdef HAVE_POPEN
-#ifndef POPEN_MODE
-#define POPEN_MODE "r"
-#endif
-    f = popen(identifier + 1, POPEN_MODE);
     *io_type = lsx_io_pipe;
+#if HAVE_POPEN
+    f = popen(identifier + 1, POPEN_MODE);
+    if (f) {
+      incr_pipe_size(f);
+    }
 #else
     lsx_fail("this build of SoX cannot open pipes");
 #endif
-    return f;
+  } else if (is_url(identifier)) {
+      *io_type = lsx_io_url;
+      f = open_url(identifier);
+  } else {
+      *io_type = lsx_io_file;
+      f = lsx_fopen(identifier, mode);
   }
-  else if (is_url(identifier)) {
-    FILE * f = NULL;
-#ifdef HAVE_POPEN
-    char const * const command_format = "wget --no-check-certificate -q -O- \"%s\"";
-    char * command = lsx_malloc(strlen(command_format) + strlen(identifier));
-    sprintf(command, command_format, identifier);
-    f = popen(command, POPEN_MODE);
-    free(command);
-    *io_type = lsx_io_url;
-#else
-    lsx_fail("this build of SoX cannot open URLs");
-#endif
-    return f;
-  }
-  return fopen(identifier, mode);
-}
-
-/* Hack to rewind pipes (a small amount).
- * Works by resetting the FILE buffer pointer */
-static void UNUSED rewind_pipe(FILE * fp)
-{
-/* _FSTDIO is for Torek stdio (i.e. most BSD-derived libc's)
- * In theory, we no longer need to check _NEWLIB_VERSION or __APPLE__ */
-#if defined _FSTDIO || defined _NEWLIB_VERSION || defined __APPLE__
-  fp->_p -= PIPE_AUTO_DETECT_SIZE;
-  fp->_r += PIPE_AUTO_DETECT_SIZE;
-#elif (defined __GLIBC__ || defined __HAIKU__) && ! defined __UCLIBC__
-  fp->_IO_read_ptr = fp->_IO_read_base;
-#elif defined sun
-# define NO_REWIND_PIPE
-#elif (defined _MSC_VER && _MSC_VER < 1900 && !defined _UCRT) || defined _WIN32 || defined _WIN64 || \
-      defined _ISO_STDIO_ISO_H || defined __sgi
-  fp->_ptr = fp->_base;
-#else
-  /* Either live without file-type detection with pipes,
-   * or add support for your compiler in the lines above.
-   * Test with cat monkey.wav | ./sox --info - */
-# define NO_REWIND_PIPE
-#endif
-  (void)fp;
+  return f;
 }
 
 static sox_format_t * open_read(
@@ -445,6 +818,7 @@ static sox_format_t * open_read(
   char const * type = "";
   size_t   input_bufsiz = sox_globals.input_bufsiz?
       sox_globals.input_bufsiz : sox_globals.bufsiz;
+  ft->pending_count = 0;
 
   if (filetype) {
     if (!(handler = sox_find_format(filetype, sox_false))) {
@@ -472,30 +846,21 @@ static sox_format_t * open_read(
         xfopen(path, "rb", &ft->io_type);
       type = io_types[ft->io_type];
       if (ft->fp == NULL) {
-        lsx_fail("can't open input %s `%s': %s", type, path, strerror(errno));
+        /* Pipe and URL openers will already have emitted an error message */
+        if (strcmp(type, "file") == 0)
+          lsx_fail("can't open input file `%s': %s", path, strerror(errno));
         goto error;
       }
     }
     if (setvbuf (ft->fp, NULL, _IOFBF, sizeof(char) * input_bufsiz)) {
-      lsx_fail("Can't set read buffer");
+      lsx_fail("can't set read buffer");
       goto error;
     }
     ft->seekable = is_seekable(ft);
   }
 
   if (!filetype) {
-    if (ft->seekable) {
-      filetype = auto_detect_format(ft, lsx_find_file_extension(path));
-      lsx_rewind(ft);
-    }
-#ifndef NO_REWIND_PIPE
-    else if (!(ft->handler.flags & SOX_FILE_NOSTDIO) &&
-        input_bufsiz >= PIPE_AUTO_DETECT_SIZE) {
-      filetype = auto_detect_format(ft, lsx_find_file_extension(path));
-      rewind_pipe(ft->fp);
-      ft->tell_off = 0;
-    }
-#endif
+    filetype = auto_detect_format(ft, lsx_find_file_extension(path));
 
     if (filetype) {
       lsx_report("detected file format type `%s'", filetype);
@@ -858,7 +1223,7 @@ static sox_format_t * open_write(
     }
     else {
       struct stat st;
-      if (!stat(path, &st) && (st.st_mode & S_IFMT) == S_IFREG &&
+      if (!lsx_stat(path, &st) && (st.st_mode & S_IFMT) == S_IFREG &&
           (overwrite_permitted && !overwrite_permitted(path))) {
         lsx_fail("permission to overwrite `%s' denied", path);
         goto error;
@@ -868,7 +1233,7 @@ static sox_format_t * open_write(
         buffer? fmemopen(buffer, buffer_size, "w+b") :
         buffer_ptr? open_memstream(buffer_ptr, buffer_size_ptr) :
 #endif
-        fopen(path, "w+b");
+        lsx_fopen(path, "w+b");
       if (ft->fp == NULL) {
         lsx_fail("can't open output file `%s': %s", path, strerror(errno));
         goto error;
@@ -878,7 +1243,7 @@ static sox_format_t * open_write(
     /* stdout tends to be line-buffered.  Override this */
     /* to be Full Buffering. */
     if (setvbuf (ft->fp, NULL, _IOFBF, sizeof(char) * sox_globals.bufsiz)) {
-      lsx_fail("Can't set write buffer");
+      lsx_fail("can't set write buffer");
       goto error;
     }
     ft->seekable = is_seekable(ft);
@@ -1088,7 +1453,7 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
     *slash_pos = '\0';
 
   if (file == NULL) {
-    lsx_fail("Can't open playlist file `%s': %s", listname, strerror(errno));
+    lsx_fail("can't open playlist file `%s': %s", listname, strerror(errno));
     result = SOX_EOF;
   }
   else {
@@ -1156,7 +1521,7 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
 /*----------------------------- Formats library ------------------------------*/
 
 enum {
-  #define FORMAT(f) f,
+  #define FORMAT(f) fmt_ ## f,
   #include "formats.h"
   #undef FORMAT
   NSTATIC_FORMATS
@@ -1197,7 +1562,7 @@ sox_get_format_fns(void)
   {
     lt_dlhandle lth = lt_dlopenext(file);
     const char *end = file + strlen(file);
-    const char prefix[] = "sox_ng_fmt_";
+    const char prefix[] = "sox_fmt_";
     char fnname[MAX_NAME_LEN];
     char *start = strstr(file, prefix);
 
