@@ -19,10 +19,6 @@
  */
 
 #include "sox_i.h"
-#include <limits.h>
-#include <string.h>
-#include <sys/stat.h>
-#include <stdarg.h>
 
 void lsx_fail_errno(sox_format_t * ft, int sox_errno, const char *fmt, ...)
 {
@@ -107,13 +103,121 @@ int lsx_check_read_params(sox_format_t * ft, unsigned channels,
  */
 size_t lsx_readbuf(sox_format_t * ft, void *buf, size_t len)
 {
-  size_t ret = fread(buf, (size_t) 1, len, (FILE*)ft->fp);
+  size_t ret;
+  size_t bytes_read = 0;
+
+  if (ft->pending_count) {
+    bytes_read = min(len, ft->pending_count);
+    memcpy(buf, ft->pending_bytes, bytes_read);
+    ft->pending_bytes += bytes_read;
+    ft->pending_count -= bytes_read;
+    if (ft->pending_count == 0)
+      free(ft->pending_buffer);
+  }
+
+  clearerr((FILE*)ft->fp);  /* So that we can read again from a file being written */
+
+  ret = bytes_read;
+  if (bytes_read < len) {
+    size_t new = fread((char *)buf + bytes_read, (size_t) 1, len - bytes_read, (FILE*)ft->fp);
+    ret += new;
+  }
+
   if (ret != len && ferror((FILE*)ft->fp))
     lsx_fail_errno(ft, errno, "lsx_readbuf");
   ft->tell_off += ret;
   return ret;
 }
 
+/* Stuff a buffer of characters back up the input stream,
+ * a similar idea to stdio's ungetc().
+ * read(a) read(b) unread(b) unread(a) should be a no-op
+ * and a following read should return a then b,
+ * so if there is already pending input, add the new characters
+ * before the existing ones.
+ *
+ * Always succeeds.
+ */
+void lsx_unreadbuf(sox_format_t * ft, void *buf, size_t len)
+{
+  if (len == 0) return;
+
+  /* If there is no pending buffer, allocate it afresh */
+  if (ft->pending_count == 0) {
+    ft->pending_buffer = lsx_malloc(len);
+    memcpy(ft->pending_buffer, buf, len);
+    ft->pending_bytes = ft->pending_buffer;
+    ft->pending_count = len;
+  } else {
+    /* If there are already some bytes waiting to be returned,
+     * add the new stuff before the existing ones.
+     */
+    /* Is there already room for them? */
+    if ((size_t)(ft->pending_bytes - ft->pending_buffer) <= len) {
+      memcpy(ft->pending_bytes -= len, buf, len);
+      ft->pending_count += len;
+    } else {
+      /* No? Dump the old buffer and make a new one
+       * with the new bytes at the start and the old ones after them.
+       */
+      sox_uint8_t *new_buffer = lsx_malloc(ft->pending_count + len);
+      memcpy(new_buffer, buf, len);
+      memcpy(new_buffer + len, ft->pending_bytes, ft->pending_count);
+      free(ft->pending_buffer);
+      ft->pending_buffer = new_buffer;
+      ft->pending_bytes += len;
+    }
+  }
+  ft->tell_off -= len;
+}
+
+/* Read in a buffer of data of length len bytes and rewind the stream.
+ * Returns number of bytes read. Unlike lsx_readbuf, it always tries to
+ * read the number of bytes that were requested.
+ */
+size_t lsx_readbuf_rewind(sox_format_t * ft, void *buf, size_t len)
+{
+  size_t ret;
+  size_t bytes_read = 0;
+
+  if (ft->pending_count) {
+    bytes_read = min(len, ft->pending_count);
+    memcpy(buf, ft->pending_bytes, bytes_read);
+    /* We don't need to "consume" them from the buffer because
+     * readbuf_rewind called twice wuold return the same again
+     */
+    if (bytes_read < len) lsx_warn("Won't be able to rewind again");
+  }
+
+  clearerr((FILE*)ft->fp);  /* So that we can read again from a file being written */
+
+  while (bytes_read < len) {
+    size_t new;
+
+    new = fread((char *)buf + bytes_read, (size_t) 1, len - bytes_read, (FILE*)ft->fp);
+    if (new == 0) /* EOF */
+      break;
+    bytes_read += new;
+  }
+
+  ret = bytes_read;
+  if (ret != len && ferror((FILE*)ft->fp))
+    lsx_fail_errno(ft, errno, "lsx_readbuf");
+
+  if (!ft->seekable) {
+    /* Remember the bytes that were returned so that we can return them again
+     * when lsx_readbuf() or lsx_readbuf_rewind() is next called.
+     */
+    ft->pending_bytes = ft->pending_buffer = lsx_malloc(bytes_read);
+    memcpy(ft->pending_bytes, buf, bytes_read);
+    ft->pending_count = bytes_read;
+  } else {
+    rewind((FILE *)ft->fp);
+  }
+  ft->tell_off = 0;
+  return ret;
+}
+
 /* Skip input without seeking. */
 int lsx_skipbytes(sox_format_t * ft, size_t n)
 {
@@ -170,6 +274,7 @@ off_t lsx_tell(sox_format_t * ft)
 
 int lsx_eof(sox_format_t * ft)
 {
+  if (ft->pending_count) return sox_false;
   return feof((FILE*)ft->fp);
 }
 
@@ -192,7 +297,9 @@ void lsx_clearerr(sox_format_t * ft)
 
 int lsx_unreadb(sox_format_t * ft, unsigned b)
 {
-  return ungetc((int)b, ft->fp);
+  unsigned char buf = b;
+  lsx_unreadbuf(ft, &buf, 1);
+  return b;
 }
 
 /* Implements traditional fseek() behavior.  Meant to abstract out
@@ -207,6 +314,16 @@ int lsx_seeki(sox_format_t * ft, off_t offset, int whence)
         if (whence != SEEK_CUR) {
             lsx_fail_errno(ft,SOX_EPERM, "file not seekable");
         } else {
+            while (offset > 0 && ft->pending_count > 0) {
+                ++ft->pending_bytes;
+                --ft->pending_count;
+                --offset;
+                ++ft->tell_off;
+            }
+            if (ft->pending_count == 0)
+                free(ft->pending_buffer);
+
+            /* If a stream peel off chars else EPERM */
             while (offset > 0 && !feof((FILE*)ft->fp)) {
                 getc((FILE*)ft->fp);
                 offset--;
@@ -359,10 +476,14 @@ static uint8_t const cswap[256] = {
   size_t lsx_read_ ## type ## _buf( \
       sox_format_t * ft, ctype *buf, size_t len) \
   { \
-    size_t n, nread; \
-    nread = lsx_readbuf(ft, buf, len * size) / size; \
+    size_t n, bytesread, nread; \
+    sox_uint8_t *rawbuf = (sox_uint8_t*)buf; \
+    bytesread = lsx_readbuf(ft, rawbuf, len * size); \
+    nread = bytesread / size; \
     for (n = 0; n < nread; n++) \
       twiddle(buf[n], type); \
+    if (bytesread > nread * size) \
+      lsx_unreadbuf(ft, rawbuf + nread * size, bytesread - nread * size); \
     return nread; \
   }
 
@@ -377,13 +498,16 @@ static uint8_t const cswap[256] = {
   size_t lsx_read_ ## type ## _buf( \
       sox_format_t * ft, ctype *buf, size_t len) \
   { \
-    size_t n, nread; \
+    int n, bytesread, nread; \
     uint8_t *data = lsx_malloc(size * len); \
-    nread = lsx_readbuf(ft, data, len * size) / size; \
+    bytesread = lsx_readbuf(ft, data, len * size); \
+    nread = bytesread / size; \
     for (n = 0; n < nread; n++) \
       buf[n] = sox_unpack ## size(data + n * size); \
+    if (bytesread > nread * size) \
+      lsx_unreadbuf(ft, data + nread * size, bytesread - nread * size); \
     free(data); \
-    return n; \
+    return nread; \
   }
 
 READ_FUNC(b, 1, uint8_t, TWIDDLE_BYTE)
