@@ -18,7 +18,6 @@
 #define LSX_EFF_ALIAS
 #include "sox_i.h"
 #include <ctype.h>
-#include <string.h>
 
 typedef struct {
   sox_bool      do_equalise, do_balance, do_balance_no_clip, do_limiter;
@@ -223,31 +222,33 @@ static int stop(sox_effect_t * effp)
 {
   priv_t * p = (priv_t *)effp->priv;
   if (p->do_scan)
-    lsx_close_tmpfile(p->tmp_file);
+    fclose(p->tmp_file); /* auto-deleted by lsx_tmpfile */
   return SOX_SUCCESS;
 }
 
 sox_effect_handler_t const * lsx_gain_effect_fn(void)
 {
-  static sox_effect_handler_t handler = {
-    "gain", NULL, SOX_EFF_GAIN,
-    create, start, flow, drain, stop, NULL, sizeof(priv_t)};
-  static char const * lines[] = {
-    "[-e|-b|-B|-r] [-n] [-l|-h] [gain-dB]",
-    "-e\t Equalise channels: peak to that with max peak;",
-    "-B\t Balance channels: rms to that with max rms; no clip protection",
-    "-b\t Balance channels: rms to that with max rms; clip protection",
-    "\t   Note -Bn = -bn",
-    "-r\t Reclaim headroom (as much as possible without clipping); see -h",
-    "-n\t Norm file to 0dBfs(output precision); gain-dB, if present, usually <0",
-    "-l\t Use simple limiter",
-    "-h\t Apply attenuation for headroom for subsequent effects; gain-dB, if",
-    "\t   present, is subject to reclaim by a subsequent gain -r",
-    "gain-dB\t Apply gain in dB",
+  static const char usage[] =
+    "[-e|-b|-B|-r] [-n] [-l|-h] [gain]";
+
+  static char const * const extra_usage[] = {
+"-e   Equalize channels: peak to that with max peak",
+"-B   Balance channels' RMS to that with max RMS; no clip protection",
+"-b   Balance channels' RMS to that with max RMS; clip protection",
+"-r   Reclaim headroom (as much as possible without clipping); see -h",
+"-n   Normalize file to 0dBfs output precision; gain, if present, usually < 0",
+"     Note: -Bn is the same as -bn",
+"-l   Use a simple limiter",
+"-h   Apply attenuation for headroom for subsequent effects;",
+"     gain, if present, is subject to reclaim by a subsequent gain -r",
+"gain Apply gain in dB",
+    NULL
   };
-  static char * usage;
-  handler.usage = lsx_usage_lines(&usage, lines, array_length(lines));
-  return &handler;
+  static sox_effect_handler_t handler = {
+    "gain", usage, extra_usage, SOX_EFF_GAIN,
+    create, start, flow, drain, stop, NULL, sizeof(priv_t)};
+
+    return &handler;
 }
 
 /*------------------ emulation of the old `normalise' effect -----------------*/
