@@ -20,10 +20,6 @@
 
 #ifdef HAVE_LADSPA_H
 
-#include <assert.h>
-#include <limits.h>
-#include <string.h>
-#include <math.h>
 #include "ladspa.h"
 
 /*
@@ -99,13 +95,6 @@ static LADSPA_Data ladspa_default(const LADSPA_PortRangeHint *p)
 /*
  * Process options
  */
-#if defined(__APPLE__) && defined(__MACH__)
-# include <TargetConditionals.h>  /* for TARGET_OS_MAC */
-#endif
-#if defined(__HAIKU__)
-# include <os/storage/FindDirectory.h>
-#endif
-
 static int sox_ladspa_getopts(sox_effect_t *effp, int argc, char **argv)
 {
   priv_t * l_st = (priv_t *)effp->priv;
@@ -113,7 +102,6 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int argc, char **argv)
   int c;
   union {LADSPA_Descriptor_Function fn; lt_ptr ptr;} ltptr;
   unsigned long index = 0, i;
-  char *newpath = NULL;
   double arg;
   lsx_getopt_t optstate;
   lsx_getopt_init(argc, argv, "+rl", NULL, lsx_getopt_flag_none, 1, &optstate);
@@ -131,77 +119,19 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int argc, char **argv)
   if (argc >= 1) {
     l_st->name = argv[0];
     argc--; argv++;
-  } else {
-    /* The module name is mandatory */
-    return lsx_usage(effp);
   }
 
   /* Load module */
-
-  /* Get the user-defined path first so it has priority */
   path = getenv("LADSPA_PATH");
   if (path == NULL)
     path = LADSPA_PATH;
 
-#if defined(unix) || defined(__unix__) || defined(__unix)
-  /* If --prefix=/usr/local, LADSPA_PATH will be /usr/local/lib/ladspa
-   * but most plugins are in /usr/lib/ladspa
-   */
-  {
-    char extra[] = "/usr/lib/ladspa";
-    if (strstr(path, extra) == NULL) {
-      newpath = lsx_realloc(newpath, strlen(path) + 1 + strlen(extra) + 1);
-      sprintf(newpath, "%s:%s", path, extra);
-      path = newpath;
-    }
-  }
-#endif
-
-#if defined(__APPLE__) && defined(__MACH__)
-# if TARGET_OS_MAC == 1
-  /* MacOS X */
-  {
-    char extra[] = "/Library/Audio/Plug-Ins/LADSPA";
-    if (strstr(path, extra) == NULL) {
-      newpath = lsx_realloc(newpath, strlen(path) + 1 + strlen(extra) + 1);
-      sprintf(newpath, "%s:%s", path, extra);
-      path = newpath;
-    }
-  }
-# endif
-#endif
-
-#if defined(__HAIKU__)
-  {
-    char **paths;
-    size_t pathCount;
-    status_t status;
-
-    status = find_paths(B_FIND_PATH_ADD_ONS_DIRECTORY, "LADSPA", &paths, &pathCount);
-    if (status == 0) {
-      while (pathCount > 0) {
-        char *extra = *paths++;
-        if (strstr(path, extra) == NULL) {
-          newpath = lsx_realloc(newpath, strlen(path) + 1 + strlen(extra) + 1);
-          sprintf(newpath, "%s:%s", path, extra);
-          path = newpath;
-	}
-	pathCount--;
-      }
-      free(paths):
-    }
-  }
-#endif
-
   if(lt_dlinit() || lt_dlsetsearchpath(path)
       || (l_st->lth = lt_dlopenext(l_st->name)) == NULL) {
     lsx_fail("could not open LADSPA plugin %s", l_st->name);
     return SOX_EOF;
   }
 
-  /* lt_dlsetsearchpath() copies path into its own static */
-  free(newpath);
-
   /* Get descriptor function */
   if ((ltptr.ptr = lt_dlsym(l_st->lth, "ladspa_descriptor")) == NULL) {
     lsx_fail("could not find ladspa_descriptor");
@@ -231,9 +161,9 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int argc, char **argv)
   }
 
   /* Scan the ports for inputs and outputs */
-  l_st->control = lsx_calloc(l_st->desc->PortCount, sizeof(LADSPA_Data));
-  l_st->inputs = lsx_malloc(l_st->desc->PortCount * sizeof(unsigned long));
-  l_st->outputs = lsx_malloc(l_st->desc->PortCount * sizeof(unsigned long));
+  lsx_vcalloc(l_st->control, l_st->desc->PortCount);
+  lsx_valloc(l_st->inputs,  l_st->desc->PortCount);
+  lsx_valloc(l_st->outputs, l_st->desc->PortCount);
 
   for (i = 0; i < l_st->desc->PortCount; i++) {
     const LADSPA_PortDescriptor port = l_st->desc->PortDescriptors[i];
@@ -398,12 +328,14 @@ static int sox_ladspa_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sa
   const size_t total_output_count = l_st->output_count * l_st->handle_count;
   const size_t channel_len = min(*isamp / total_input_count, *osamp / total_output_count + l_st->in_latency);
 
-  LADSPA_Data *buf = lsx_calloc(channel_len * total_input_count, sizeof(LADSPA_Data));
-  LADSPA_Data *outbuf = lsx_calloc(channel_len * total_output_count, sizeof(LADSPA_Data));
+  LADSPA_Data *buf, *outbuf;
   LADSPA_Handle handle;
   unsigned long port, l;
   SOX_SAMPLE_LOCALS;
 
+  lsx_vcalloc(buf, channel_len * total_input_count);
+  lsx_vcalloc(outbuf, channel_len * total_output_count);
+
   *isamp = channel_len * total_input_count;
   *osamp = 0;
 
@@ -484,7 +416,7 @@ static int sox_ladspa_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osa
 
   /* feed some silence at the end to push the rest of the data out */
   isamp = l_st->out_latency * effp->in_signal.channels;
-  ibuf = lsx_calloc(isamp, sizeof(sox_sample_t));
+  lsx_vcalloc(ibuf, isamp);
 
   do {
     dsamp = min(l_st->out_latency * effp->out_signal.channels,
@@ -537,7 +469,7 @@ static int sox_ladspa_kill(sox_effect_t * effp)
 
 static sox_effect_handler_t sox_ladspa_effect = {
   "ladspa",
-  "MODULE [PLUGIN] [ARGUMENT...]",
+  "MODULE [PLUGIN] [ARGUMENT...]", NULL, 
   SOX_EFF_MCHAN | SOX_EFF_CHAN | SOX_EFF_GAIN,
   sox_ladspa_getopts,
   sox_ladspa_start,
