@@ -56,7 +56,6 @@
  */
 
 #include "sox_i.h"
-#include <string.h>
 
 #if defined(HAVE_LAME_LAME_H) || defined(HAVE_LAME_H) || defined(DL_LAME)
 #define HAVE_LAME 1
@@ -90,6 +89,7 @@ typedef enum {
 
 #ifdef USING_ID3TAG
   #include <id3tag.h>
+  #include "id3.h"
 #if defined(HAVE_UNISTD_H)
   #include <unistd.h>
 #elif defined(HAVE_IO_H)
@@ -123,12 +123,12 @@ static mad_timer_t const mad_timer_zero_stub = {0, 0};
 #define ID3PADDING 128
 
 /* LAME takes float values as input. */
-#define MP3_LAME_PRECISION 24
+#define MP3_LAME_PRECISION   24
 
 /* MAD returns values with MAD_F_FRACBITS (28) bits of precision, though it's
    not certain that all of them are meaningful. Default to 16 bits to
    align with most users expectation of output file should be 16 bits. */
-#define MP3_MAD_PRECISION  16
+#define MP3_MAD_PRECISION    16
 
 #ifdef HAVE_MAD_H
 static const char* const mad_library_names[] =
@@ -140,7 +140,6 @@ static const char* const mad_library_names[] =
 #endif
     NULL
 };
-#endif
 
 #ifdef DL_MAD
   #define MAD_FUNC LSX_DLENTRY_DYNAMIC
@@ -166,8 +165,9 @@ static const char* const mad_library_names[] =
   MAD_FUNC(f,x, void, mad_header_init, (struct mad_header *)) \
   MAD_FUNC(f,x, signed long, mad_timer_count, (mad_timer_t, enum mad_units)) \
   MAD_FUNC(f,x, void, mad_timer_multiply, (mad_timer_t *, signed long))
+#endif /* HAVE_MAD_H */
 
-#ifdef HAVE_LAME
+#if HAVE_LAME
 static const char* const lame_library_names[] =
 {
 #ifdef DL_LAME
@@ -244,6 +244,7 @@ static const char* const twolame_library_names[] =
 #endif
   NULL
 };
+#endif /* HAVE_TWOLAME_H */
 
 #ifdef DL_TWOLAME
   #define TWOLAME_FUNC LSX_DLENTRY_DYNAMIC
@@ -262,7 +263,6 @@ static const char* const twolame_library_names[] =
   TWOLAME_FUNC(f,x, int, twolame_encode_buffer_float32_interleaved, (twolame_options *, const float [], int, unsigned char *, int)) \
   TWOLAME_FUNC(f,x, int, twolame_encode_flush, (twolame_options *, unsigned char *, int)) \
   TWOLAME_FUNC(f,x, void, twolame_close, (twolame_options **))
-#endif /* HAVE_TWOLANE_H */
 
 /* Private data */
 typedef struct mp3_priv_t {
@@ -298,7 +298,7 @@ typedef struct mp3_priv_t {
 #endif
 } priv_t;
 
-#ifdef HAVE_MAD_H
+#if defined(HAVE_MAD_H)
 
 /* This function merges the functions tagtype() and id3_tag_query()
    from MAD's libid3tag, so we don't have to link to it
@@ -329,7 +329,7 @@ static int tagtype(const unsigned char *data, size_t length)
     return 0;
 }
 
-#endif /*HAVE_MAD_H*/
+#endif /* HAVE_MAD_H */
 
 #include "mp3-util.h"
 
@@ -340,7 +340,7 @@ static int tagtype(const unsigned char *data, size_t length)
  * still exists in the buffer then they are first shifted to be
  * front of the stream buffer.
  */
-static int sox_mp3_input(sox_format_t * ft)
+static int sox_mad_input(sox_format_t * ft)
 {
     priv_t *p = (priv_t *) ft->priv;
     size_t bytes_read;
@@ -377,7 +377,7 @@ static int sox_mp3_input(sox_format_t * ft)
  * consume it all.  Returns SOX_EOF if no tag is found.  Its up to
  * caller to recover.
  * */
-static int sox_mp3_inputtag(sox_format_t * ft)
+static int sox_mad_inputtag(sox_format_t * ft)
 {
     priv_t *p = (priv_t *) ft->priv;
     int rc = SOX_EOF;
@@ -435,7 +435,7 @@ static int startread(sox_format_t * ft)
   ft->signal.length = SOX_UNSPEC;
   if (ft->seekable) {
 #ifdef USING_ID3TAG
-    read_comments(ft);
+    lsx_id3_read_tag(ft, sox_true);
     lsx_rewind(ft);
     if (!ft->signal.length)
 #endif
@@ -470,14 +470,14 @@ static int startread(sox_format_t * ft)
       /* check whether input buffer needs a refill */
       if (p->Stream.error == MAD_ERROR_BUFLEN)
       {
-          if (sox_mp3_input(ft) == SOX_EOF)
+          if (sox_mad_input(ft) == SOX_EOF)
               return SOX_EOF;
 
           continue;
       }
 
       /* Consume any ID3 tags */
-      sox_mp3_inputtag(ft);
+      sox_mad_inputtag(ft);
 
       /* FIXME: We should probably detect when we've read
        * a bunch of non-ID3 data and still haven't found a
@@ -489,7 +489,7 @@ static int startread(sox_format_t * ft)
 
   if (p->Stream.error)
   {
-      lsx_fail_errno(ft,SOX_EOF,"No valid MP3 frame found");
+      lsx_fail_errno(ft,SOX_EOF,"no valid MP3 frame found");
       return SOX_EOF;
   }
 
@@ -502,7 +502,7 @@ static int startread(sox_format_t * ft)
           ft->signal.channels = MAD_NCHANNELS(&p->Frame.header);
           break;
       default:
-          lsx_fail_errno(ft, SOX_EFMT, "Cannot determine number of channels");
+          lsx_fail_errno(ft, SOX_EFMT, "cannot determine number of channels");
           return SOX_EOF;
   }
 
@@ -562,8 +562,8 @@ static size_t sox_mp3read(sox_format_t * ft, sox_sample_t *buf, size_t len)
         /* check whether input buffer needs a refill */
         if (p->Stream.error == MAD_ERROR_BUFLEN)
         {
-            if (sox_mp3_input(ft) == SOX_EOF) {
-                lsx_debug("sox_mp3_input EOF");
+            if (sox_mad_input(ft) == SOX_EOF) {
+                lsx_debug("sox_mad_input EOF");
                 break;
             }
         }
@@ -572,7 +572,7 @@ static size_t sox_mp3read(sox_format_t * ft, sox_sample_t *buf, size_t len)
         {
             if(MAD_RECOVERABLE(p->Stream.error))
             {
-                sox_mp3_inputtag(ft);
+                sox_mad_inputtag(ft);
                 continue;
             }
             else
@@ -640,7 +640,7 @@ static int sox_mp3seek(sox_format_t * ft, sox_uint64_t offset)
     size_t read;
     size_t leftover = p->Stream.bufend - p->Stream.next_frame;
 
-    memmove(p->mp3_buffer, p->Stream.this_frame, leftover);
+    memcpy(p->mp3_buffer, p->Stream.this_frame, leftover);
     read = lsx_readbuf(ft, p->mp3_buffer + leftover, p->mp3_buffer_size - leftover);
     if (read == 0) {
       lsx_debug("seek failure. unexpected EOF (frames=%" PRIuPTR " leftover=%" PRIuPTR ")", p->FrameCount, leftover);
@@ -714,7 +714,11 @@ static int sox_mp3seek(sox_format_t * ft, sox_uint64_t offset)
 
   return SOX_EOF;
 }
-#else /* !HAVE_MAD_H */
+#endif /* !HAVE_MAD_H */
+
+
+
+#ifndef HAVE_MAD_H
 static int startread(sox_format_t * ft)
 {
   lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 decoding support");
@@ -829,11 +833,6 @@ static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, uint64_t num
   }
 
   buffer = lsx_malloc(id3v2_size);
-  if (!buffer)
-  {
-    lsx_warn("cannot update track length info - failed to allocate buffer");
-    return;
-  }
 
   if (num_samples > ULONG_MAX)
   {
@@ -987,7 +986,7 @@ static int startwrite(sox_format_t * ft)
     p->opt = p->twolame_init();
 
     if (p->opt == NULL){
-      lsx_fail_errno(ft,SOX_EOF,"Initialization of Twolame library failed");
+      lsx_fail_errno(ft,SOX_EOF,"initialization of Twolame library failed");
       return(SOX_EOF);
     }
 #endif
@@ -996,7 +995,7 @@ static int startwrite(sox_format_t * ft)
     p->gfp = p->lame_init();
 
     if (p->gfp == NULL){
-      lsx_fail_errno(ft,SOX_EOF,"Initialization of LAME library failed");
+      lsx_fail_errno(ft,SOX_EOF,"initialization of LAME library failed");
       return(SOX_EOF);
     }
 
@@ -1023,7 +1022,7 @@ static int startwrite(sox_format_t * ft)
 #endif
     }
     if (fail) {
-      lsx_fail_errno(ft,SOX_EOF,"Unsupported number of channels");
+      lsx_fail_errno(ft,SOX_EOF,"unsupported number of channels");
       return(SOX_EOF);
     }
   }
@@ -1110,7 +1109,7 @@ static int startwrite(sox_format_t * ft)
 
     if (ft->encoding.compression < 0.5) {
       if (p->mp2) {
-        lsx_fail_errno(ft,SOX_EOF,"Variable bitrate encoding not supported for MP2 audio");
+        lsx_fail_errno(ft,SOX_EOF,"variable bitrate encoding not supported for MP2 audio");
         return(SOX_EOF);
       }
 #ifdef HAVE_LAME
@@ -1206,10 +1205,6 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
     new_buffer_size = samp * sizeof(float);
     if (p->pcm_buffer_size < new_buffer_size) {
       float *new_buffer = lsx_realloc(p->pcm_buffer, new_buffer_size);
-      if (!new_buffer) {
-        lsx_fail_errno(ft, SOX_ENOMEM, "Out of memory");
-        return 0;
-      }
       p->pcm_buffer_size = new_buffer_size;
       p->pcm_buffer = new_buffer;
     }
@@ -1249,10 +1244,6 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
     new_buffer_size = LAME_BUFFER_SIZE(nsamples);
     if (p->mp3_buffer_size < new_buffer_size) {
       unsigned char *new_buffer = lsx_realloc(p->mp3_buffer, new_buffer_size);
-      if (!new_buffer) {
-        lsx_fail_errno(ft, SOX_ENOMEM, "Out of memory");
-        return 0;
-      }
       p->mp3_buffer_size = new_buffer_size;
       p->mp3_buffer = new_buffer;
     }
@@ -1269,13 +1260,13 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
 #endif
     }
     if (written < 0) {
-        lsx_fail_errno(ft,SOX_EOF,"Encoding failed");
+        lsx_fail_errno(ft,SOX_EOF,"encoding failed");
         return 0;
     }
 
     if (lsx_writebuf(ft, p->mp3_buffer, (size_t)written) < (size_t)written)
     {
-        lsx_fail_errno(ft,SOX_EOF,"File write failed");
+        lsx_fail_errno(ft,SOX_EOF,"file write failed");
         return 0;
     }
 
@@ -1300,9 +1291,9 @@ static int stopwrite(sox_format_t * ft)
 #endif
   }
   if (written < 0)
-    lsx_fail_errno(ft, SOX_EOF, "Encoding failed");
+    lsx_fail_errno(ft, SOX_EOF, "encoding failed");
   else if (lsx_writebuf(ft, p->mp3_buffer, (size_t)written) < (size_t)written)
-    lsx_fail_errno(ft, SOX_EOF, "File write failed");
+    lsx_fail_errno(ft, SOX_EOF, "file write failed");
   else if (!p->mp2) {
 #ifdef HAVE_LAME
     if (ft->seekable && (num_samples != p->num_samples || p->vbr_tag))
