@@ -30,7 +30,6 @@
 
 #include "sox_i.h"
 
-#include <string.h>
 
 /* Private data for silence effect. */
 
@@ -118,7 +117,7 @@ static int sox_silence_getopts(sox_effect_t * effp, int argc, char **argv)
       return lsx_usage(effp);
     if (silence->start_periods < 0)
     {
-        lsx_fail("Periods must not be negative");
+        lsx_fail("periods must not be negative");
         return(SOX_EOF);
     }
     argv++;
@@ -198,18 +197,18 @@ static int sox_silence_getopts(sox_effect_t * effp, int argc, char **argv)
     {
         if ((silence->start_unit != '%') && (silence->start_unit != 'd'))
         {
-            lsx_fail("Invalid unit specified");
+            lsx_fail("invalid unit specified");
             return lsx_usage(effp);
         }
         if ((silence->start_unit == '%') && ((silence->start_threshold < 0.0)
             || (silence->start_threshold > 100.0)))
         {
-            lsx_fail("silence threshold should be between 0.0 and 100.0 %%");
+            lsx_fail("threshold should be between 0.0 and 100.0 %%");
             return (SOX_EOF);
         }
         if ((silence->start_unit == 'd') && (silence->start_threshold >= 0.0))
         {
-            lsx_fail("silence threshold should be less than 0.0 dB");
+            lsx_fail("threshold should be less than 0.0 dB");
             return(SOX_EOF);
         }
     }
@@ -218,18 +217,18 @@ static int sox_silence_getopts(sox_effect_t * effp, int argc, char **argv)
     {
         if ((silence->stop_unit != '%') && (silence->stop_unit != 'd'))
         {
-            lsx_fail("Invalid unit specified");
+            lsx_fail("invalid unit specified");
             return(SOX_EOF);
         }
         if ((silence->stop_unit == '%') && ((silence->stop_threshold < 0.0) ||
                     (silence->stop_threshold > 100.0)))
         {
-            lsx_fail("silence threshold should be between 0.0 and 100.0 %%");
+            lsx_fail("threshold should be between 0.0 and 100.0 %%");
             return (SOX_EOF);
         }
         if ((silence->stop_unit == 'd') && (silence->stop_threshold >= 0.0))
         {
-            lsx_fail("silence threshold should be less than 0.0 dB");
+            lsx_fail("threshold should be less than 0.0 dB");
             return(SOX_EOF);
         }
     }
@@ -243,11 +242,11 @@ static int sox_silence_start(sox_effect_t * effp)
 
     /* When you want to remove silence, small window sizes are
      * better or else RMS will look like non-silence at
-     * aburpt changes from load to silence.
+     * abrupt changes from load to silence.
      */
     silence->window_size = (effp->in_signal.rate / 50) * 
         effp->in_signal.channels;
-    silence->window = lsx_malloc(silence->window_size * sizeof(double));
+    lsx_valloc(silence->window, silence->window_size);
 
     clear_rms(effp);
 
@@ -272,12 +271,12 @@ static int sox_silence_start(sox_effect_t * effp)
     else
         silence->mode = SILENCE_COPY;
 
-    silence->start_holdoff = lsx_malloc(sizeof(sox_sample_t)*silence->start_duration);
+    lsx_valloc(silence->start_holdoff, silence->start_duration);
     silence->start_holdoff_offset = 0;
     silence->start_holdoff_end = 0;
     silence->start_found_periods = 0;
 
-    silence->stop_holdoff = lsx_malloc(sizeof(sox_sample_t)*silence->stop_duration);
+    lsx_valloc(silence->stop_holdoff, silence->stop_duration);
     silence->stop_holdoff_offset = 0;
     silence->stop_holdoff_end = 0;
     silence->stop_found_periods = 0;
@@ -689,7 +688,7 @@ static int lsx_kill(sox_effect_t * effp)
 
 static sox_effect_handler_t sox_silence_effect = {
   "silence",
-  "[ -l ] above_periods [ duration threshold[d|%] ] [ below_periods duration threshold[d|%] ]",
+  "[-l] above_periods [duration threshold[d|%]] [below_periods duration threshold[d|%]]", NULL,
   SOX_EFF_MCHAN | SOX_EFF_MODIFY | SOX_EFF_LENGTH,
   sox_silence_getopts,
   sox_silence_start,
