@@ -168,11 +168,9 @@ static sox_encoding_t sox_enc(int ft_encoding, unsigned * size)
     case SF_FORMAT_DWVW_24  : *size = 24; return SOX_ENCODING_DWVW;
     case SF_FORMAT_DWVW_N   : *size =  0; return SOX_ENCODING_DWVWN;
     case SF_FORMAT_GSM610   : *size =  0; return SOX_ENCODING_GSM;
-#ifdef SF_FORMAT_MPEG_LAYER_II
+#if HAVE_SF_FORMAT_MPEG
     case SF_FORMAT_MPEG_LAYER_II
                             : *size = 16; return SOX_ENCODING_MP3;
-#endif
-#ifdef SF_FORMAT_MPEG_LAYER_III
     case SF_FORMAT_MPEG_LAYER_III
                             : *size = 16; return SOX_ENCODING_MP3;
 #endif
