@@ -14,17 +14,17 @@
 #define SOX_I_H
 
 #include "soxomp.h"  /* Note: soxomp.h includes soxconfig.h */
-#include "sox_ng.h"
 
 #if defined HAVE_FMEMOPEN
 #define _GNU_SOURCE
 #endif
 
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-
+#include "sox_ng.h"
 #include "util.h"
+#include "win32-unicode.h"
+
+#include <assert.h>
+#include <errno.h>
 
 #if defined(LSX_EFF_ALIAS)
 #undef lsx_debug
@@ -40,7 +40,7 @@
 #define RANQD1 ranqd1(sox_globals.ranqd1)
 #define DRANQD1 dranqd1(sox_globals.ranqd1)
 
-typedef enum {SOX_SHORT, SOX_INT, SOX_FLOAT, SOX_DOUBLE} sox_data_t;
+typedef enum {SOX_SHORT, SOX_INT, SOX_FLOAT, SOX_DOUBLE, SOX_SAMPLE} sox_data_t;
 typedef enum {SOX_WAVE_SINE, SOX_WAVE_TRIANGLE} lsx_wave_t;
 lsx_enum_item const * lsx_get_wave_enum(void);
 
@@ -55,7 +55,6 @@ assert_static(sizeof(off_t) == _FILE_OFFSET_BITS >> 3, OFF_T_BUILD_PROBLEM);
 #endif
 
 FILE * lsx_tmpfile(void);
-void   lsx_close_tmpfile(FILE *);
 
 void lsx_debug_more_impl(char const * fmt, ...) LSX_PRINTF12;
 void lsx_debug_most_impl(char const * fmt, ...) LSX_PRINTF12;
@@ -141,6 +140,8 @@ void lsx_load_samples(double * const dest, sox_sample_t const * const src,
 
 /* Read and write basic data types from "ft" stream. */
 size_t lsx_readbuf(sox_format_t * ft, void *buf, size_t len);
+void   lsx_unreadbuf(sox_format_t * ft, void *buf, size_t len);
+size_t lsx_readbuf_rewind(sox_format_t * ft, void *buf, size_t len);
 int lsx_skipbytes(sox_format_t * ft, size_t n);
 int lsx_padbytes(sox_format_t * ft, size_t n);
 size_t lsx_writebuf(sox_format_t * ft, void const *buf, size_t len);
@@ -174,15 +175,8 @@ int lsx_readqw(sox_format_t * ft, uint64_t * udw);
 int lsx_readf(sox_format_t * ft, float * f);
 int lsx_readw(sox_format_t * ft, uint16_t * uw);
 
-#if 1 /* FIXME: use defines */
-UNUSED static int lsx_readsb(sox_format_t * ft, int8_t * sb)
-{return lsx_readb(ft, (uint8_t *)sb);}
-UNUSED static int lsx_readsw(sox_format_t * ft, int16_t * sw)
-{return lsx_readw(ft, (uint16_t *)sw);}
-#else
 #define lsx_readsb(ft, sb) lsx_readb(ft, (uint8_t *)sb)
-#define lsx_readsw(ft, sw) lsx_readb(ft, (uint16_t *)sw)
-#endif
+#define lsx_readsw(ft, sw) lsx_readw(ft, (uint16_t *)sw)
 
 int lsx_write3(sox_format_t * ft, unsigned u3);
 int lsx_writeb(sox_format_t * ft, unsigned ub);
@@ -244,6 +238,9 @@ int lsx_rawstart(sox_format_t * ft, sox_bool default_rate, sox_bool default_chan
 #define lsx_rawstopwrite NULL
 
 extern sox_format_handler_t const * lsx_sndfile_format_fn(void);
+#if HAVE_POPEN
+extern sox_format_handler_t const * lsx_ffmpeg_format_fn(void);
+#endif
 
 char * lsx_cat_comments(sox_comments_t comments);
 
@@ -252,7 +249,6 @@ char * lsx_cat_comments(sox_comments_t comments);
 int lsx_flow_copy(sox_effect_t * effp, const sox_sample_t * ibuf,
     sox_sample_t * obuf, size_t * isamp, size_t * osamp);
 int lsx_usage(sox_effect_t * effp);
-char * lsx_usage_lines(char * * usage, char const * const * lines, size_t n);
 #define EFFECT(f) extern sox_effect_handler_t const * lsx_##f##_effect_fn(void);
 #include "effects.h"
 #undef EFFECT
@@ -265,7 +261,7 @@ char * lsx_usage_lines(char * * usage, char const * const * lines, size_t n);
   if (end_ptr != *argv) { \
     if (d < min || d > max || *end_ptr != '\0') {\
       lsx_fail("parameter `%s' must be from %g to %g", #name, (double)min, (double)max); \
-      return SOX_EOF; \
+      return lsx_usage(effp); \
     } \
     p->name = d; \
     --argc, ++argv; \
@@ -287,7 +283,7 @@ char * lsx_usage_lines(char * * usage, char const * const * lines, size_t n);
   double d = strtod(state.arg, &end_ptr); \
   if (end_ptr == state.arg || d < min || d > max || *end_ptr != '\0') {\
     lsx_fail("parameter `%s' must be from %g to %g", #name, (double)min, (double)max); \
-    return SOX_EOF; \
+    return lsx_usage(effp); \
   } \
   name = d; \
   break; \
