@@ -157,8 +157,6 @@ BLOCK 9 - data block that supersedes blocks 1 and 8.
 #include "sox_i.h"
 #include "g711.h"
 #include "adpcms.h"
-#include <assert.h>
-#include <string.h>
 
 /* Private data for VOC file */
 typedef struct {
@@ -220,11 +218,11 @@ static int startread(sox_format_t * ft)
   unsigned char uc;
 
   if (lsx_readbuf(ft, header, (size_t)20) != 20) {
-    lsx_fail_errno(ft, SOX_EHDR, "unexpected EOF in VOC header");
+    lsx_fail_errno(ft, SOX_EHDR, "unexpected EOF in header");
     return (SOX_EOF);
   }
   if (strncmp(header, "Creative Voice File\032", (size_t)19)) {
-    lsx_fail_errno(ft, SOX_EHDR, "VOC file header incorrect");
+    lsx_fail_errno(ft, SOX_EHDR, "file header incorrect");
     return (SOX_EOF);
   }
 
@@ -255,7 +253,7 @@ static int startread(sox_format_t * ft)
 
   /* get rate of data */
   if (v->rate == -1) {
-    lsx_fail_errno(ft, SOX_EOF, "Input .voc file had no sound!");
+    lsx_fail_errno(ft, SOX_EOF, "input file had no sound!");
     return (SOX_EOF);
   }
 
@@ -296,7 +294,8 @@ static int startread(sox_format_t * ft)
       v->size = 4;
       break;
     default:
-      lsx_fail("Unknown VOC format %d", v->format);
+      lsx_fail("unknown format %d", v->format);
+      return (SOX_EOF);
       break;
   }
   ft->encoding.bits_per_sample = v->size;
@@ -331,6 +330,11 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t * buf,
   if (v->block_remaining == 0)  /* if no more data, return 0, i.e., done */
     return 0;
 
+  if (v->size == 0) {
+    lsx_warn("VOC input: zero file size");
+    return 0;
+  }
+
   if (v->silent) {
     for (; v->block_remaining && (done < len); v->block_remaining--, done++)
       *buf++ = 0;       /* Fill in silence */
@@ -352,8 +356,8 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t * buf,
       if (v->size <= 4) {
         if (!v->adpcm.setup.sign) {
           SOX_SAMPLE_LOCALS;
-          if (lsx_readb(ft, &uc) == SOX_EOF) {
-            lsx_warn("VOC input: short file");
+          if (lsx_readb(ft, &uc)) {
+            lsx_warn("short input file");
             v->block_remaining = 0;
             return done;
           }
@@ -368,8 +372,8 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t * buf,
           --v->block_remaining;
           ++done;
         }
-        if (lsx_readb(ft, &uc) == SOX_EOF) {
-          lsx_warn("VOC input: short file");
+        if (lsx_readb(ft, &uc)) {
+          lsx_warn("short input file");
           v->block_remaining = 0;
           return done;
         }
@@ -414,8 +418,8 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t * buf,
       } else
         switch (v->size) {
           case 8:
-            if (lsx_readb(ft, &uc) == SOX_EOF) {
-              lsx_warn("VOC input: short file");
+            if (lsx_readb(ft, &uc)) {
+              lsx_warn("short input file");
               v->block_remaining = 0;
               return done;
             }
@@ -428,8 +432,7 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t * buf,
             }
             break;
           case 16:
-            if (lsx_readsw(ft, &sw) == SOX_EOF ||
-	        lsx_eof(ft)) {
+            if (lsx_readsw(ft, &sw) || lsx_eof(ft)) {
               lsx_warn("short input file");
               v->block_remaining = 0;
               return done;
@@ -466,7 +469,7 @@ static int startwrite(sox_format_t * ft)
 
   if (!ft->seekable) {
     lsx_fail_errno(ft, SOX_EOF,
-                   "Output .voc file must be a file, not a pipe");
+                   "Output must be a file, not a pipe");
     return (SOX_EOF);
   }
 
@@ -604,7 +607,7 @@ static int getblock(sox_format_t * ft)
         /* block, the DATA blocks rate value is invalid */
         if (!v->extended) {
           if (uc == 0) {
-            lsx_fail_errno(ft, SOX_EFMT, "Sample rate is zero?");
+            lsx_fail_errno(ft, SOX_EFMT, "sample rate is zero");
             return (SOX_EOF);
           }
           if ((v->rate != -1) && (uc != v->rate)) {
@@ -627,7 +630,7 @@ static int getblock(sox_format_t * ft)
         if (lsx_readdw(ft, &new_rate_32))
 	  return SOX_EOF;
         if (new_rate_32 == 0) {
-          lsx_fail_errno(ft, SOX_EFMT, "Sample rate is zero?");
+          lsx_fail_errno(ft, SOX_EFMT, "sample rate is zero");
           return (SOX_EOF);
         }
         if ((v->rate != -1) && ((long) new_rate_32 != v->rate)) {
@@ -661,7 +664,7 @@ static int getblock(sox_format_t * ft)
               lsx_readb(ft, &uc))
 	    return SOX_EOF;
           if (uc == 0) {
-            lsx_fail_errno(ft, SOX_EFMT, "Silence sample rate is zero");
+            lsx_fail_errno(ft, SOX_EFMT, "silence sample rate is zero");
             return (SOX_EOF);
           }
           /*
@@ -723,7 +726,7 @@ voc_text:
         if (lsx_readw(ft, &new_rate_16))
 	  return SOX_EOF;
         if (new_rate_16 == 0) {
-          lsx_fail_errno(ft, SOX_EFMT, "Sample rate is zero?");
+          lsx_fail_errno(ft, SOX_EFMT, "sample rate is zero");
           return (SOX_EOF);
         }
         if ((v->rate != -1) && (new_rate_16 != v->rate)) {
