@@ -27,9 +27,6 @@
 
 #include "sox_i.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
 
 #include "ima_rw.h"
 #include "adpcm.h"
@@ -151,7 +148,7 @@ static unsigned short  ImaAdpcmReadBlock(sox_format_t * ft)
         samplesThisBlock = lsx_ima_samples_in((size_t)0, (size_t)ft->signal.channels, bytesRead, (size_t) 0);
         if (samplesThisBlock == 0 || samplesThisBlock > wav->samplesPerBlock)
         {
-            lsx_warn("Premature EOF on .wav input file");
+            lsx_warn("Premature EOF on input file");
             return 0;
         }
     }
@@ -192,7 +189,7 @@ static unsigned short  AdpcmReadBlock(sox_format_t * ft)
         samplesThisBlock = lsx_ms_adpcm_samples_in((size_t)0, (size_t)ft->signal.channels, bytesRead, (size_t)0);
         if (samplesThisBlock == 0 || samplesThisBlock > wav->samplesPerBlock)
         {
-            lsx_warn("Premature EOF on .wav input file");
+            lsx_warn("Premature EOF on input file");
             return 0;
         }
     }
@@ -262,7 +259,7 @@ static int wavgsminit(sox_format_t * ft)
         return (SOX_EOF);
     }
 
-    wav->gsmsample=lsx_malloc(sizeof(gsm_signal)*160*2);
+    lsx_valloc(wav->gsmsample, 160*2);
     wav->gsmindex=0;
     return (SOX_SUCCESS);
 }
@@ -295,7 +292,7 @@ static size_t wavgsmread(sox_format_t * ft, sox_sample_t *buf, size_t len)
         if (bytes <=0)
             return done;
         if (bytes<65) {
-            lsx_warn("invalid wav gsm frame size: %d bytes",bytes);
+            lsx_warn("invalid GSM frame size: %d bytes",bytes);
             return done;
         }
         /* decode the long 33 byte half */
@@ -416,7 +413,7 @@ static int sndfile_workaround(uint64_t *len, sox_format_t *ft) {
     /* Seek back to where we were, which won't work if you're piping */
     if (lsx_seeki(ft, here, SEEK_SET)!=SOX_SUCCESS)
     {
-        lsx_fail_errno(ft,SOX_EHDR, "Cannot seek backwards to work around possible broken header.");
+        lsx_fail_errno(ft,SOX_EHDR, "cannot seek backwards to work around possible broken header");
         return SOX_EOF;
     }
     if (memcmp(magic, "fmt ", (size_t)4)==0)
@@ -460,7 +457,7 @@ static int findChunk(sox_format_t * ft, const char *Label, uint64_t *len)
             }
             else
             {
-                lsx_fail_errno(ft, SOX_EHDR, "Cannot yet read block sizes of arbitrary RF64 chunks, cannot find chunk '%s'", Label);
+                lsx_fail_errno(ft, SOX_EHDR, "cannot yet read block sizes of arbitrary RF64 chunks, cannot find chunk '%s'", Label);
                 return SOX_EOF;
             }
         }
@@ -490,7 +487,7 @@ static int findChunk(sox_format_t * ft, const char *Label, uint64_t *len)
         if (*len > 0 && lsx_seeki(ft, (off_t)(*len), SEEK_CUR) != SOX_SUCCESS)
         {
             lsx_fail_errno(ft,SOX_EHDR,
-                          "WAV chunk appears to have invalid size %" PRIu64 ".", *len);
+                          "chunk appears to have invalid size %" PRIu64 ".", *len);
             return SOX_EOF;
         }
     }
@@ -500,7 +497,7 @@ static int findChunk(sox_format_t * ft, const char *Label, uint64_t *len)
 
 static int wavfail(sox_format_t * ft, const char *format)
 {
-    lsx_fail_errno(ft, SOX_EHDR, "WAV file encoding `%s' is not supported", format);
+    lsx_fail_errno(ft, SOX_EHDR, "file encoding `%s' is not supported", format);
     return SOX_EOF;
 }
 
@@ -548,7 +545,7 @@ static int startread(sox_format_t * ft)
          strncmp("RIFX", magic, (size_t)4) != 0 &&
 	 strncmp("RF64", magic, (size_t)4)!=0 ))
     {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE: RIFF header not found");
+        lsx_fail_errno(ft,SOX_EHDR,"RIFF header not found");
         return SOX_EOF;
     }
 
@@ -602,14 +599,14 @@ static int startread(sox_format_t * ft)
     /* Now look for the format chunk */
     if (findChunk(ft, "fmt ", &len) == SOX_EOF)
     {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE chunk fmt not found");
+        lsx_fail_errno(ft,SOX_EHDR,"fmt chunk not found");
         return SOX_EOF;
     }
     wFmtSize = len;
 
     if (wFmtSize < 16)
     {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE file fmt chunk is too short");
+        lsx_fail_errno(ft,SOX_EHDR,"fmt chunk is too short");
         return SOX_EOF;
     }
 
@@ -633,7 +630,7 @@ static int startread(sox_format_t * ft)
 
       if (wFmtSize < 18)
       {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE file fmt chunk is too short");
+        lsx_fail_errno(ft,SOX_EHDR,"fmt chunk is too short");
         return SOX_EOF;
       }
       if (lsx_readw(ft, &extensionSize))
@@ -641,7 +638,7 @@ static int startread(sox_format_t * ft)
       len -= 2;
       if (extensionSize < 22)
       {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE file fmt chunk is too short");
+        lsx_fail_errno(ft,SOX_EHDR,"fmt chunk is too short");
         return SOX_EOF;
       }
       if (lsx_readw(ft, &numberOfValidBits) ||
@@ -652,9 +649,9 @@ static int startread(sox_format_t * ft)
         if (lsx_readb(ft, &dummyByte))
           read_error();
       len -= 22;
-      if (numberOfValidBits != wBitsPerSample)
+      if (numberOfValidBits > wBitsPerSample)
       {
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE file fmt with padded samples is not supported yet");
+        lsx_fail_errno(ft,SOX_EHDR,"number of valid bits exceeds the number of bits per sample");
         return SOX_EOF;
       }
       wav->formatTag = subFormatTag;
@@ -664,7 +661,7 @@ static int startread(sox_format_t * ft)
     switch (wav->formatTag)
     {
     case WAVE_FORMAT_UNKNOWN:
-        lsx_fail_errno(ft,SOX_EHDR,"WAVE file is in unsupported Microsoft Official Unknown format.");
+        lsx_fail_errno(ft,SOX_EHDR,"file is in unsupported Microsoft Official Unknown format");
         return SOX_EOF;
 
     case WAVE_FORMAT_PCM:
@@ -740,7 +737,7 @@ static int startread(sox_format_t * ft)
     case WAVE_FORMAT_G722_ADPCM:
         return wavfail(ft, "G.722 ADPCM");
     default:
-        lsx_fail_errno(ft, SOX_EHDR, "Unknown WAV file encoding (type %x)", wav->formatTag);
+        lsx_fail_errno(ft, SOX_EHDR, "unknown file encoding (type %x)", wav->formatTag);
         return SOX_EOF;
     }
 
@@ -751,7 +748,7 @@ static int startread(sox_format_t * ft)
         lsx_report("User options overriding channels read in .wav header");
 
     if (ft->signal.channels == 0) {
-        lsx_fail_errno(ft, SOX_EHDR, "Channel count is zero");
+        lsx_fail_errno(ft, SOX_EHDR, "channel count is zero");
         return SOX_EOF;
     }
 
@@ -782,7 +779,7 @@ static int startread(sox_format_t * ft)
 
     if (wExtSize > len)
     {
-        lsx_fail_errno(ft,SOX_EOF,"wave header error: wExtSize inconsistent with wFmtLen");
+        lsx_fail_errno(ft,SOX_EOF,"header error: wExtSize inconsistent with wFmtLen");
         return SOX_EOF;
     }
 
@@ -798,7 +795,7 @@ static int startread(sox_format_t * ft)
 
         if (wBitsPerSample != 4)
         {
-            lsx_fail_errno(ft,SOX_EOF,"Can only handle 4-bit MS ADPCM in wav files");
+            lsx_fail_errno(ft,SOX_EOF,"can only handle 4-bit MS ADPCM");
             return SOX_EOF;
         }
 
@@ -824,14 +821,14 @@ static int startread(sox_format_t * ft)
 
         if (wExtSize < 4 + 4*wav->nCoefs)
         {
-            lsx_fail_errno(ft,SOX_EOF,"wave header error: wExtSize(%d) too small for nCoefs(%d)", wExtSize, wav->nCoefs);
+            lsx_fail_errno(ft,SOX_EOF,"header error: wExtSize(%d) too small for nCoefs(%d)", wExtSize, wav->nCoefs);
             return SOX_EOF;
         }
 
-        wav->samples = lsx_malloc(wChannels*wav->samplesPerBlock*sizeof(short));
+        lsx_valloc(wav->samples, wChannels * wav->samplesPerBlock);
 
         /* nCoefs, lsx_ms_adpcm_i_coefs used by adpcm.c */
-        wav->lsx_ms_adpcm_i_coefs = lsx_malloc(wav->nCoefs * 2 * sizeof(short));
+        lsx_valloc(wav->lsx_ms_adpcm_i_coefs, wav->nCoefs * 2);
         wav->ms_adpcm_data = lsx_ms_adpcm_alloc(wChannels);
         {
             int i, errct=0;
@@ -858,7 +855,7 @@ static int startread(sox_format_t * ft)
 
         if (wBitsPerSample != 4)
         {
-            lsx_fail_errno(ft,SOX_EOF,"Can only handle 4-bit IMA ADPCM in wav files");
+            lsx_fail_errno(ft,SOX_EOF,"can only handle 4-bit IMA ADPCM");
             return SOX_EOF;
         }
 
@@ -875,7 +872,7 @@ static int startread(sox_format_t * ft)
         wav->packet = lsx_malloc((size_t)wav->blockAlign);
         len -= 2;
 
-        wav->samples = lsx_malloc(wChannels*wav->samplesPerBlock*sizeof(short));
+        lsx_valloc(wav->samples, wChannels * wav->samplesPerBlock);
 
         bytespersample = 2;  /* AFTER de-compression */
         break;
@@ -916,7 +913,7 @@ static int startread(sox_format_t * ft)
     if (!ft->encoding.bits_per_sample || ft->encoding.bits_per_sample == wBitsPerSample)
       ft->encoding.bits_per_sample = wBitsPerSample;
     else
-      lsx_warn("User options overriding size read in .wav header");
+      lsx_warn("User options overriding size in header");
 
     /* Now we have enough information to set default encodings. */
     switch (bytespersample)
@@ -937,7 +934,7 @@ static int startread(sox_format_t * ft)
       break;
 
     default:
-      lsx_fail_errno(ft,SOX_EFMT,"Sorry, don't understand .wav size");
+      lsx_fail_errno(ft,SOX_EFMT,"don't understand size");
       return SOX_EOF;
     }
 
@@ -951,7 +948,7 @@ static int startread(sox_format_t * ft)
     /* Now look for the wave data chunk */
     if (findChunk(ft, "data", &len) == SOX_EOF)
     {
-        lsx_fail_errno(ft, SOX_EOF, "Could not find data chunk.");
+        lsx_fail_errno(ft, SOX_EOF, "could not find data chunk");
         return SOX_EOF;
     }
 
@@ -1000,7 +997,7 @@ static int startread(sox_format_t * ft)
     default:
         if (ft->encoding.bits_per_sample == 0)
         {
-            lsx_fail_errno(ft, SOX_EHDR, "WAV file bits per sample is zero");
+            lsx_fail_errno(ft, SOX_EHDR, "bits per sample is zero");
             return SOX_EOF;
         }
         wav->numSamples = div_bits(qwDataLength, ft->encoding.bits_per_sample) / ft->signal.channels;
@@ -1238,7 +1235,7 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
 
             done = wavgsmread(ft, buf, len);
             if (done == 0 && wav->numSamples != 0 && !wav->ignoreSize)
-                lsx_warn("Premature EOF on .wav input file");
+                lsx_warn("Premature EOF on input file");
         break;
 
         default: /* assume PCM or float encoding */
@@ -1249,7 +1246,7 @@ static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
             /* If software thinks there are more samples but I/O */
             /* says otherwise, let the user know about this.     */
             if (done == 0 && wav->numSamples != 0 && !wav->ignoreSize)
-                lsx_warn("Premature EOF on .wav input file");
+                lsx_warn("Premature EOF on input file");
         }
 
         /* Only return buffers that contain a totally playable
@@ -1313,7 +1310,7 @@ static int startwrite(sox_format_t * ft)
     wav->numSamples = 0;
     wav->dataLength = 0;
     if (!ft->signal.length && !ft->seekable)
-        lsx_warn("Length in output .wav header will be wrong since can't seek to fix it");
+        lsx_warn("Length in output header will be wrong since can't seek to fix it");
 
     rc = wavwritehdr(ft, 0);  /* also calculates various wav->* info */
     if (rc != 0)
@@ -1337,7 +1334,7 @@ wave_format_adpcm:
                 wav->state[ch] = 0;
             sbsize = ft->signal.channels * wav->samplesPerBlock;
             wav->packet = lsx_malloc((size_t)wav->blockAlign);
-            wav->samples = lsx_malloc(sbsize*sizeof(short));
+            lsx_valloc(wav->samples, sbsize);
             wav->sampleTop = wav->samples + sbsize;
             wav->samplePtr = wav->samples;
             break;
@@ -1431,19 +1428,19 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
 
     /* fact chunk (not PCM) */
     uint32_t dwFactSize=4;        /* length of the fact chunk */
-    uint64_t dwSamplesWritten=0;  /* windows doesnt seem to use this*/
+    uint32_t dwSamplesWritten=0;  /* windows doesnt seem to use this*/
 
     /* data chunk */
-    uint64_t  dwDataLength; /* length of sound data in bytes */
+    uint32_t  dwDataLength; /* length of sound data in bytes */
     /* end of variables written to header */
 
     /* internal variables, intermediate values etc */
     int bytespersample; /* (uncompressed) bytes per sample (per channel) */
-    uint64_t blocksWritten = 0;
+    long blocksWritten = 0;
     sox_bool isExtensible = sox_false;    /* WAVE_FORMAT_EXTENSIBLE? */
 
     if (ft->signal.channels > UINT16_MAX) {
-        lsx_fail_errno(ft, SOX_EOF, "Too many channels (%u)",
+        lsx_fail_errno(ft, SOX_EOF, "too many channels (%u)",
                        ft->signal.channels);
         return SOX_EOF;
     }
@@ -1477,7 +1474,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         case SOX_ENCODING_IMA_ADPCM:
             if (wChannels>16)
             {
-                lsx_fail_errno(ft,SOX_EOF,"Channels(%d) must be <= 16",wChannels);
+                lsx_fail_errno(ft,SOX_EOF,"channels(%d) must be <= 16",wChannels);
                 return SOX_EOF;
             }
             wFormatTag = WAVE_FORMAT_IMA_ADPCM;
@@ -1489,7 +1486,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         case SOX_ENCODING_MS_ADPCM:
             if (wChannels>16)
             {
-                lsx_fail_errno(ft,SOX_EOF,"Channels(%d) must be <= 16",wChannels);
+                lsx_fail_errno(ft,SOX_EOF,"channels(%d) must be <= 16",wChannels);
                 return SOX_EOF;
             }
             wFormatTag = WAVE_FORMAT_ADPCM;
@@ -1525,7 +1522,8 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
      * hint then write default value.  Also, use default value even
      * on header update if more then 32-bit length needs to be written.
      */
-    if (!second_header && !ft->signal.length) {
+    if ((!second_header && !ft->signal.length) || 
+        wav->numSamples > 0xffffffff) { 
         /* adjust for blockAlign */
         blocksWritten = MS_UNSPEC/wBlockAlign;
         dwDataLength = blocksWritten * wBlockAlign;
@@ -1538,7 +1536,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
     }
 
     if (wFormatTag == WAVE_FORMAT_GSM610)
-        if (dwDataLength & 1) dwDataLength++; /* round up to even */
+        dwDataLength = (dwDataLength+1) & ~1u; /* round up to even */
 
     if (wFormatTag == WAVE_FORMAT_PCM && (wBitsPerSample > 16 || wChannels > 2)
         && strcmp(ft->filetype, "wavpcm")) {
@@ -1553,10 +1551,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         wRiffLength += (8+dwFactSize);
 
     /* dwAvgBytesPerSec <-- this is BEFORE compression, isn't it? guess not. */
-    /* Round before dividing so that txw's 33333.3 doesn't become
-     * 33333 with 66667 byte rate.
-     */
-    dwAvgBytesPerSec = (wBlockAlign*lrint(ft->signal.rate)) / wSamplesPerBlock;
+    dwAvgBytesPerSec = (double)wBlockAlign*ft->signal.rate / (double)wSamplesPerBlock + 0.5;
 
     /* figured out header info, so write it */
 
@@ -1641,27 +1636,16 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
         break;
     }
 
-    /* WAV files can't speciy lengths more than 4G samples or 4GB of data:
-     * warn and write UNSPEC instead of creating files of a random size. */
-    if (dwSamplesWritten > 0xffffffff) {
-        lsx_warn("length exceeds 4G samples: file may read truncated");
-	dwSamplesWritten = MS_UNSPEC;
-    }
-    if (dwDataLength > 0xffffffff) {
-        lsx_warn("length exceeds 4GB of data: file may read truncated");
-	dwDataLength = MS_UNSPEC;
-    }
-
     /* if not PCM, write the 'fact' chunk */
     if (isExtensible || wFormatTag != WAVE_FORMAT_PCM){
         if (lsx_writes(ft, "fact") ||
             lsx_writedw(ft,dwFactSize) ||
-            lsx_writedw(ft,(uint32_t)dwSamplesWritten))
+            lsx_writedw(ft,dwSamplesWritten))
 	        write_error();
     }
 
     if (lsx_writes(ft, "data") ||
-        lsx_writedw(ft, (uint32_t)dwDataLength))     /* data chunk size */
+        lsx_writedw(ft, dwDataLength))               /* data chunk size */
 	    write_error();
 
     if (!second_header) {
@@ -1672,13 +1656,13 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
                 dwAvgBytesPerSec, wBlockAlign, wBitsPerSample);
     } else {
         lsx_debug("Finished writing Wave file, %u data bytes %lu samples",
-                (uint32_t)dwDataLength, (unsigned long)wav->numSamples);
+                dwDataLength, (unsigned long)wav->numSamples);
         if (wFormatTag == WAVE_FORMAT_GSM610){
-            lsx_debug("GSM6.10 format: %u blocks %u padded samples %u padded data bytes",
-                    (uint32_t)blocksWritten, (uint32_t)dwSamplesWritten, (uint32_t)dwDataLength);
+            lsx_debug("GSM6.10 format: %li blocks %u padded samples %u padded data bytes",
+                    blocksWritten, dwSamplesWritten, dwDataLength);
             if (wav->gsmbytecount != dwDataLength)
                 lsx_warn("help ! internal inconsistency - data_written %u gsmbytecount %lu",
-                        (uint32_t)dwDataLength, (unsigned long)wav->gsmbytecount);
+                        dwDataLength, (unsigned long)wav->gsmbytecount);
 
         }
     }
@@ -1767,7 +1751,7 @@ static int stopwrite(sox_format_t * ft)
 
         if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
         {
-                lsx_fail_errno(ft,SOX_EOF,"Can't rewind output file to rewrite .wav header.");
+                lsx_fail_errno(ft,SOX_EOF,"can't rewind output file to rewrite header");
                 return SOX_EOF;
         }
 
